@@ -222,7 +222,7 @@ message_viewer_wire_section_footer(message_viewer *mv, u32 section_idx, u16 coun
 
 
 static void
-message_viewer_wire_question_record(message_viewer *mv, u8 *record_wire, u16 rclass, u16 rtype)
+message_viewer_wire_question_record(message_viewer *mv, const u8 *record_wire, u16 rclass, u16 rtype)
 {
     if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
@@ -247,7 +247,7 @@ message_viewer_wire_question_record(message_viewer *mv, u8 *record_wire, u16 rcl
     u64 next = counters.write_count + 24 + 8;
 
     /* write NAME + alignment for next item */
-    osformat(os, ";%{dnsname}", record_wire, ' ');
+    osformat(os, ";%{dnsname}", record_wire);
     while(counters.write_count < next)
     {
         output_stream_write_u8(os, (u8) ' ');
@@ -272,7 +272,7 @@ message_viewer_wire_question_record(message_viewer *mv, u8 *record_wire, u16 rcl
 
 
 static void
-message_viewer_wire_section_record(message_viewer *mv, u8 *record_wire, u8 section_idx)
+message_viewer_wire_section_record(message_viewer *mv, const u8 *record_wire, u8 section_idx)
 {
     (void)section_idx;
 
@@ -292,8 +292,8 @@ message_viewer_wire_section_record(message_viewer *mv, u8 *record_wire, u8 secti
 
 
     /* 1. get the needed parameters: FQDN, TYPE, CLASS, TTL, RDATA size */
-    u8 *rname      = record_wire;
-    u8 *rdata      = rname + dnsname_len(rname);
+    const u8 *rname      = record_wire;
+    const u8 *rdata      = rname + dnsname_len(rname);
     u16 rtype      = GET_U16_AT(rdata[0]);
     u16 rclass     = GET_U16_AT(rdata[2]);
     u32 rttl       = ntohl(GET_U32_AT(rdata[4]));
@@ -357,6 +357,12 @@ message_viewer_wire_section_record(message_viewer *mv, u8 *record_wire, u8 secti
     flushout();
 }
 
+static ya_result
+message_viewer_wire_pseudosection_record_method(message_viewer *mv, const u8 *record_wire)
+{
+    return 0;
+}
+
 
 static const message_viewer_vtbl wire_viewer_vtbl = {
        message_viewer_wire_header,
@@ -366,6 +372,7 @@ static const message_viewer_vtbl wire_viewer_vtbl = {
        message_viewer_wire_section_footer,
        message_viewer_wire_question_record,
        message_viewer_wire_section_record,
+       message_viewer_wire_pseudosection_record_method,
        "message_viewer_wire",
 };
 
