@@ -46,6 +46,8 @@
 #ifndef _DEBUG_H
 #define	_DEBUG_H
 
+#include <dnscore/dnscore-config-features.h>
+
 #ifndef _SYSTYPES_H
 #error PLEASE DO NOT INCLUDE debug.h DIRECTLY.  USE sys_types.h.
 #endif
@@ -53,20 +55,33 @@
 #define DEBUG_STAT_SIZES 1 // Common
 #define DEBUG_STAT_TAGS  2 // Usefull
 #define DEBUG_STAT_DUMP  4 // USE WITH CARE
-
-#if DEBUG
-#include <dnscore/thread.h>
-#endif
+#define DEBUG_STAT_WALK  8
+#define DEBUG_STAT_MMAP 16
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
 /** @note : DO NOT INCLUDE THIS HERE WITHOUT SOME KIND OF PROTECTION #include <dnscore/format.h> */
 
-
-
 #include <dnscore/debug_config.h>
 
+#if DNSCORE_HAS_MMAP_DEBUG_SUPPORT
+// MUST be included before else the defines will break the build
+#include <sys/mman.h>
+
+//#pragma message("MMAP REROUTED")
+void* debug_mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t off);
+int debug_munmap(void *addr, size_t len);
+void debug_mmap_stat();
+
+#define mmap(addr__,len__,prot__,flags__,fildes__,off__) debug_mmap((addr__),(len__),(prot__),(flags__),(fildes__),(off__))
+#define munmap(addr__,len__) debug_munmap((addr__),(len__))
+#endif
+
+#if DEBUG
+#include <dnscore/thread.h>
+#endif
+
 #if !DEBUG
 #define yassert(x)
 #else
@@ -114,6 +129,8 @@ bool debug_log_stacktrace(struct logger_handle *handle, u32 level, const char *p
 #error "bogus include sequence"
 #endif
 
+void debug_stat(int mask);
+
 #if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT
 
 #ifndef MALLOC_OR_DIE
@@ -155,7 +172,6 @@ bool debug_mallocated(void* ptr);
 void debug_free(void* ptr,const char* file, int line);
 void debug_mtest(void* ptr);
 
-void debug_stat(int mask);
 char* debug_strdup(const char*);
 
 u32 debug_get_block_count();
@@ -168,18 +184,18 @@ u32 debug_get_block_count();
 
 #if !DNSCORE_DEBUG_HAS_BLOCK_TAG
 
-#define malloc(len) debug_malloc((len),__FILE__,__LINE__)
-#define calloc(len) debug_calloc((len),__FILE__,__LINE__)
-#define free(p) debug_free((p),__FILE__,__LINE__)
-#define realloc(p,len) debug_realloc((p),(len),__FILE__,__LINE__)
+#define malloc(len__) debug_malloc((len__),__FILE__,__LINE__)
+#define calloc(len__) debug_calloc((len__),__FILE__,__LINE__)
+#define free(p__) debug_free((p__),__FILE__,__LINE__)
+#define realloc(p__,len__) debug_realloc((p__),(len__),__FILE__,__LINE__)
 
 #define MALLOC_OR_DIE(cast,target,size,tag) if(((target)=(cast)debug_malloc(size,__FILE__,__LINE__))==NULL){perror(__FILE__);abort(); /* NOT TAGGED*/ }
 #define REALLOC_OR_DIE(cast,src_and_target,newsize,tag) if(((src_and_target)=(cast)debug_realloc((src_and_target),(newsize),__FILE__,__LINE__))==NULL){perror(__FILE__);abort(); /* NOT TAGGED */ }
 #else
 
-#define ZDB_MALLOC_TAG	0x434f4c4c414d
+#define DBGALLOC_TAG 0x434f4c4c41474244
 
-#define malloc(len__) debug_malloc((len__),__FILE__,__LINE__,ZDB_MALLOC_TAG)
+#define malloc(len__) debug_malloc((len__),__FILE__,__LINE__,DBGALLOC_TAG)
 #define free(p__) debug_free((p__),__FILE__,__LINE__)
 #define realloc(p__,len__) debug_realloc((p__),(len__),__FILE__,__LINE__)
 
@@ -190,10 +206,13 @@ u32 debug_get_block_count();
 #else
 
 #define debug_mtest(x)
-#define debug_stat(x)
 #define debug_mallocated(x) TRUE
 #define assert_mallocated(x)
 
+#if !(DNSCORE_HAS_MALLOC_DEBUG_SUPPORT || DNSCORE_HAS_ZALLOC_DEBUG_SUPPORT || DNSCORE_HAS_ZALLOC_STATISTICS_SUPPORT || DNSCORE_HAS_MMAP_DEBUG_SUPPORT)
+#define debug_stat(x)
+#endif
+
 /*
  * MALLOC_OR_DIE and REALLOC_OR_DIE have already been defined in sys_types.h
  */
@@ -212,6 +231,9 @@ struct debug_bench_s
 
 typedef struct debug_bench_s debug_bench_s;
 
+struct output_stream;
+struct logger_handle;
+
 // declares timeus()
 
 s64 timeus();
@@ -225,14 +247,12 @@ void debug_bench_register(debug_bench_s *bench, const char *name);
 #define debug_bench_stop(bench__, from__) debug_bench_commit((bench__), timeus() - (from__));
 
 void debug_bench_commit(debug_bench_s *bench, u64 delta);
-
 void debug_bench_logdump_all();
+void debug_bench_print_all(struct output_stream *os);
 
 void debug_bench_unregister_all();
 
 typedef intptr* stacktrace;
-struct output_stream;
-struct logger_handle;
 stacktrace debug_stacktrace_get();
 void debug_stacktrace_log(struct logger_handle *handle, u32 level, stacktrace trace);
 void debug_stacktrace_log_with_prefix(struct logger_handle* handle, u32 level, stacktrace trace, const char *prefix);
@@ -260,6 +280,22 @@ void *debug_malloc_unmonitored(size_t size);
 void debug_free_unmonitored(void* ptr);
 void debug_malloc_hook_tracked_dump();
 void debug_malloc_hook_caller_dump();
+void *debug_mmap(void *addr, size_t len, int prot, int flags, int fildes, off_t off);
+
+void debug_nop_hook(); // meant
+
+struct debug_memory_by_tag_context_s;
+typedef struct debug_memory_by_tag_context_s debug_memory_by_tag_context_t;
+
+#if DNSCORE_DEBUG_HAS_BLOCK_TAG
+debug_memory_by_tag_context_t* debug_memory_by_tag_new_instance(const char* name);
+void debug_memory_by_tag_delete(debug_memory_by_tag_context_t *ctx);
+void debug_memory_by_tag_init(debug_memory_by_tag_context_t *ctx, const char* name);
+void debug_memory_by_tag_finalize(debug_memory_by_tag_context_t *ctx);
+void debug_memory_by_tag_alloc_notify(debug_memory_by_tag_context_t *ctx, u64 tag, s64 size);
+void debug_memory_by_tag_free_notify(debug_memory_by_tag_context_t *ctx, u64 tag, s64 size);
+void debug_memory_by_tag_print(debug_memory_by_tag_context_t *ctx, struct output_stream *os);
+#endif
 
 #ifdef	__cplusplus
 }
