@@ -44,7 +44,7 @@
 
 #define DNSSEC_ERROR_BASE		        0x80050000
 #define DNSSEC_ERROR_CODE(code_)	    ((s32)(DNSSEC_ERROR_BASE+(code_)))
-    
+
 #define DNSSEC_ERROR_NOENGINE			DNSSEC_ERROR_CODE(  1)
 #define DNSSEC_ERROR_INVALIDENGINE		DNSSEC_ERROR_CODE(  2)
 #define DNSSEC_ERROR_CANTPOOLTHREAD		DNSSEC_ERROR_CODE(  4)
@@ -99,4 +99,7 @@
 #define DNSSEC_ERROR_RRSIG_NOSIGNINGKEY		DNSSEC_ERROR_CODE(516)
 #define DNSSEC_ERROR_RRSIG_UNSUPPORTEDRECORD	DNSSEC_ERROR_CODE(517)
 
+#define SSL_ERROR_BASE		            0x800f0000
+#define SSL_ERROR_CODE(code_)	    ((s32)(SSL_ERROR_BASE+(code_)))
+
 #endif // _DNSSEC_ERROR_H
