@@ -119,6 +119,8 @@ typedef struct logger_handle logger_handle;
 #define LOGGER_HANDLE_NAME_SIZE_MAX 16
 #define LOGGER_HANDLE_FORMATTED_NAME_SIZE_MAX 8
 
+#define LOGGER_REPEAT_COMPRESSION 1
+
 struct logger_handle // ~ 5 * 64
 {
     u8 active[MSG_LEVEL_COUNT]; // 16 /// @note 20200818 edf -- I know what I always say about layout.  This is an exception.  Keep it first.
@@ -151,7 +153,7 @@ struct logger_channel
 {
     void *data;
     const struct logger_channel_vtbl *vtbl;
-#if !HAS_SHARED_QUEUE_SUPPORT
+#if LOGGER_REPEAT_COMPRESSION
     struct logger_message *last_message;
     u32 last_message_count;
 #endif
@@ -474,6 +476,7 @@ static inline bool log_is_set(logger_handle* handle, int level)
 #define log_info(...)   LOGGER_EARLY_CULL_PREFIX(MSG_INFO) logger_handle_msg_nocull(MODULE_MSG_HANDLE,MSG_INFO,LOG_TEXT_PREFIX __VA_ARGS__)
 #define log_warn(...)   LOGGER_EARLY_CULL_PREFIX(MSG_WARNING) logger_handle_msg_nocull(MODULE_MSG_HANDLE,MSG_WARNING,LOG_TEXT_PREFIX __VA_ARGS__)
 #define log_err(...)    LOGGER_EARLY_CULL_PREFIX(MSG_ERR) logger_handle_msg_nocull(MODULE_MSG_HANDLE,MSG_ERR,LOG_TEXT_PREFIX __VA_ARGS__)
+#define log_to_level(level_,...)    LOGGER_EARLY_CULL_PREFIX((level_)) logger_handle_msg_nocull(MODULE_MSG_HANDLE,(level_),LOG_TEXT_PREFIX __VA_ARGS__)
 
 #define log_try_debug7(...) logger_handle_try_msg(MODULE_MSG_HANDLE,MSG_DEBUG7,LOG_TEXT_PREFIX __VA_ARGS__)
 #define log_try_debug6(...) logger_handle_try_msg(MODULE_MSG_HANDLE,MSG_DEBUG6,LOG_TEXT_PREFIX __VA_ARGS__)
