@@ -67,6 +67,8 @@ extern "C"
 #include <dnscore/ptr_vector.h>
 #include <dnscore/format.h>
 
+#include <dnscore/logger.h>
+
 // Processing flags
 
 #define PROCESS_FL_ADDITIONAL_AUTH      0x01
@@ -200,7 +202,7 @@ struct message_tsig
     u16 error;      /* NETWORK */
     
     u16 other_len;  /* NETWORK */
-    u16 reserved_1; /* ALIGN32 */
+    u16 tsig_offset;// keeps the tsig_offset in the message for internal processing, do not use directly
     
     u32 reserved_2; /* ALIGN64 */    
 
@@ -244,7 +246,6 @@ struct message_data
     u16 _query_class;
     bool _edns;
     bool _nsid;
-    char _protocol;         // will probably be removed
     u8 _referral;
     u8 _control_buffer_size;
     u8 _tcp_serial;
@@ -415,9 +416,9 @@ static inline u8 message_get_opcode(const message_data *mesg)
     return MESSAGE_OP(mesg->_buffer);
 }
 
-static inline void message_set_opcode(message_data *mesg, u8 op)
+static inline void message_set_opcode(message_data *mesg, u8 opcode)
 {
-    MESSAGE_SET_OP(mesg->_buffer, op);
+    MESSAGE_SET_OP(mesg->_buffer, opcode);
 }
 
 static inline void message_set_referral(message_data *mesg, u8 referral)
@@ -653,6 +654,11 @@ static inline void message_set_truncated_answer(message_data *mesg)
     MESSAGE_HIFLAGS(mesg->_buffer) |= TC_BITS|QR_BITS;
 }
 
+static inline bool message_is_truncated(message_data *mesg)
+{
+    return (MESSAGE_HIFLAGS(mesg->_buffer) & TC_BITS) != 0;
+}
+
 static inline void message_set_authoritative(message_data *mesg)
 {
     MESSAGE_HIFLAGS(mesg->_buffer) |= AA_BITS;
@@ -989,6 +995,12 @@ static inline void message_tsig_set_error(message_data *mesg, u16 err)
     mesg->_tsig.error = err;
 }
 
+static inline u16 message_tsig_get_error(message_data *mesg)
+{
+    return mesg->_tsig.error;
+}
+
+
 static inline void message_tsig_mac_copy(const message_data *mesg, u8 *to)
 {
     memcpy(to, mesg->_tsig.mac, message_tsig_mac_get_size(mesg));
@@ -1005,7 +1017,15 @@ static inline void message_tsig_copy_from(message_data *mesg, const message_data
     const message_tsig *s = &source->_tsig;
     memcpy(d, s, offsetof(message_tsig, mac));
     memcpy(d->mac, s->mac, s->mac_size);
-    d->other = s->other;
+    if((s->other != NULL) && (s->other_len > 0))
+    {
+        MALLOC_OR_DIE(u8*, d->other, s->other_len, TSIGOTHR_TAG);
+        memcpy(d->other, s->other, s->other_len);
+    }
+    else
+    {
+        d->other = NULL;
+    }
     d->hmac = s->hmac;
     d->tcp_tsig_countdown = s->tcp_tsig_countdown;
     d->mac_algorithm = s->mac_algorithm;
@@ -1065,6 +1085,7 @@ int message_process(message_data *);
 int message_process_lenient(message_data *mesg);
 
 void message_transform_to_error(message_data *mesg);
+void message_transform_to_signed_error(message_data *mesg);
 
 /* global */
 
@@ -1088,6 +1109,8 @@ void message_make_query(message_data *mesg, u16 id, const u8 *qname, u16 qtype,
 
 void message_make_query_ex(message_data *mesg, u16 id, const u8 *qname, u16 qtype, u16 qclass, u16 flags);
 
+void message_make_query_ex_with_edns0(message_data *mesg, u16 id, const u8 *qname, u16 qtype, u16 qclass, u32 edns0_ttl);
+
 struct packet_writer;
 
 void message_make_message(message_data *mesg, u16 id, const u8 *qname, u16 qtype, u16 qclass, struct packet_writer* uninitialised_packet_writer);
@@ -1120,10 +1143,14 @@ void message_make_ixfr_query(message_data *mesg, u16 id, const u8 *qname, u32 so
 
 ya_result message_sign_query_by_name(message_data *mesg, const u8 *tsig_name);
 
+ya_result message_sign_query_by_name_with_epoch_and_fudge(message_data *mesg, const u8 *tsig_name, s64 epoch, u16 fudge);
+
 ya_result message_sign_answer_by_name(message_data *mesg, const u8 *tsig_name);
 
 ya_result message_sign_query(message_data *mesg, const tsig_item *key);
 
+ya_result message_sign_query_with_epoch_and_fudge(message_data *mesg, const tsig_item *key, s64 epoch, u16 fudge);
+
 ya_result message_sign_answer(message_data *mesg, const tsig_item *key);
 
 #endif
@@ -1279,17 +1306,29 @@ static inline void message_send_udp_reset(message_data *mesg)
 }
 
 #if !DEBUG
-static inline ssize_t message_send_udp(const message_data *mesg, int sockfd)
+static inline s32 message_send_udp(const message_data *mesg, int sockfd)
 {
-    return sendmsg(sockfd, &mesg->_msghdr, 0);
+    s32 n;
+
+    while((n = sendmsg(sockfd, &mesg->_msghdr, 0)) < 0)
+    {
+        int err = errno;
+
+        if(err != EINTR)
+        {
+            return MAKE_ERRNO_ERROR(err);
+        }
+    }
+
+    return n;
 }
 #else
+s32 message_send_udp_debug(const message_data *mesg, int sockfd);
 
-ssize_t message_send_udp_debug(const message_data *mesg, int sockfd);
-
-static inline ssize_t message_send_udp(const message_data *mesg, int sockfd)
+static inline s32 message_send_udp(const message_data *mesg, int sockfd)
 {
-    return message_send_udp_debug(mesg, sockfd);
+    s32 ret = message_send_udp_debug(mesg, sockfd);
+    return ret;
 }
 #endif
 
@@ -1305,6 +1344,12 @@ static inline ssize_t message_recv_udp(message_data *mesg, int sockfd)
     if(ret >= 0)
     {
         message_set_size(mesg, ret);
+#if __FreeBSD__
+        if(mesg->_msghdr.msg_controllen == 0)
+        {
+            mesg->_msghdr.msg_control = NULL;
+        }
+#endif
     }
     return ret;
 }
@@ -1407,6 +1452,47 @@ static inline void message_set_status(message_data *mesg, finger_print fp)
     mesg->_status = fp;
 }
 
+static inline void message_set_error_status_from_result(message_data *mesg, ya_result error_code)
+{
+    finger_print fp;
+
+    if(YA_ERROR_BASE(error_code) == RCODE_ERROR_BASE)
+    {
+        fp = RCODE_ERROR_GETCODE(error_code);
+    }
+    else
+    {
+        fp = FP_RCODE_SERVFAIL;
+    }
+
+    message_set_status(mesg, fp);
+}
+
+static inline void message_set_status_from_result(message_data *mesg, ya_result error_code)
+{
+    finger_print fp;
+
+    if(ISOK(error_code))
+    {
+        fp = RCODE_NOERROR;
+    }
+    else if(YA_ERROR_BASE(error_code) == RCODE_ERROR_BASE)
+    {
+        fp = RCODE_ERROR_GETCODE(error_code);
+    }
+    else
+    {
+        fp = FP_RCODE_SERVFAIL;
+    }
+
+    message_set_status(mesg, fp);
+}
+
+static inline void message_clear_status(message_data *mesg)
+{
+    MESSAGE_FLAGS_AND(mesg->_buffer, 0xff, 0xf0);
+}
+
 static inline void message_update_answer_status(message_data *mesg)
 {
     MESSAGE_FLAGS_OR(mesg->_buffer, QR_BITS, mesg->_status);
@@ -1495,9 +1581,11 @@ static inline ssize_t message_read_tcp(message_data *mesg, input_stream *is)
     }
 }
 
+#if 0
 static inline ssize_t message_send_tcp(const message_data *mesg, int sockfd)
 {
     ssize_t ret;
+
     u16 tcp_len = htons(message_get_size_u16(mesg));
     if(ISOK(ret = writefully(sockfd, &tcp_len, 2)))
     {
@@ -1506,8 +1594,12 @@ static inline ssize_t message_send_tcp(const message_data *mesg, int sockfd)
             ret += 2;
         }
     }
+
     return ret;
 }
+#else
+ssize_t message_send_tcp(const message_data *mesg, int sockfd);
+#endif
 
 static inline ssize_t message_send_tcp_with_minimum_throughput(const message_data *mesg, int sockfd, double minimum_rate)
 {
@@ -1682,7 +1774,7 @@ static inline void message_tcp_serial_increment(message_data *mesg)
 ya_result message_query_tcp_with_timeout(message_data *mesg, const host_address *server, u8 to_sec);
 ya_result message_query_tcp_with_timeout_ex(message_data *mesg, const host_address *server, message_data *answer, u8 to_sec);
 ya_result message_query_tcp(message_data *mesg, const host_address *server);
-ya_result message_query_tcp_ex(message_data *mesg, const host_address *server, message_data *answer);
+ya_result message_query_tcp_ex(message_data *mesg, const host_address *bindto, const host_address *server, message_data *answer);
 ya_result message_query_udp(message_data *mesg, const host_address *server);
 ya_result message_query_udp_with_timeout(message_data *mesg, const host_address *server, int seconds, int useconds);
 
