@@ -145,7 +145,16 @@ bool dnscore_monitored_fail(ya_result ret);
 #define RCODE_ERROR_BASE                        0x80010000
 #define RCODE_ERROR_CODE(code_)                 ((s32)(RCODE_ERROR_BASE+(code_)))
 #define RCODE_ERROR_GETCODE(error_)             ((error_)&0xffff)
-#define MAKE_DNSMSG_ERROR(err_)                 RCODE_ERROR_CODE(err_)
+#define MAKE_DNSMSG_ERROR(err_)                 RCODE_ERROR_CODE(err_) // obsolete
+#define MAKE_RCODE_ERROR(err_)                  RCODE_ERROR_CODE(err_)
+
+#define TSIG_BADSIG                             MAKE_RCODE_ERROR(16)      /* TSIG timestamp outisde of the time window    */
+#define TSIG_BADKEY                             MAKE_RCODE_ERROR(17)      /* Unknown key name in TSIG record              */
+#define TSIG_BADTIME                            MAKE_RCODE_ERROR(18)      /* TSIG timestamp outisde of the time window    */
+#define TSIG_BADMODE                            MAKE_RCODE_ERROR(19)
+#define TSIG_BADNAME                            MAKE_RCODE_ERROR(20)
+#define TSIG_BADALG                             MAKE_RCODE_ERROR(21)
+#define TSIG_BADTRUNC                           MAKE_RCODE_ERROR(22)
 
 #define CORE_ERROR_BASE                         0x80020000
 #define CORE_ERROR_CODE(code_)                  ((s32)(CORE_ERROR_BASE+(code_)))
@@ -256,6 +265,7 @@ bool dnscore_monitored_fail(ya_result ret);
 
 #define DNS_ERROR_BASE                          0x80030000
 #define DNS_ERROR_CODE(code_)                   ((s32)(DNS_ERROR_BASE+(code_)))
+#define IS_DNS_ERROR_CODE(code_)                (((code_)&0xffff0000) == DNS_ERROR_BASE)
 #define DOMAIN_TOO_LONG                         DNS_ERROR_CODE(1)    /* FQDN is longer than 255           */
 #define INCORRECT_IPADDRESS                     DNS_ERROR_CODE(2)    /* Incorrect ip address              */
 #define INCORRECT_RDATA                         DNS_ERROR_CODE(3)
@@ -268,13 +278,6 @@ bool dnscore_monitored_fail(ya_result ret);
 #define TSIG_FORMERR                            DNS_ERROR_CODE(14)
 #define TSIG_SIZE_LIMIT_ERROR                   DNS_ERROR_CODE(15)
 
-#define TSIG_BADSIG                             DNS_ERROR_CODE(16)      /* TSIG timestamp outisde of the time window    */
-#define TSIG_BADKEY                             DNS_ERROR_CODE(17)      /* Unknown key name in TSIG record              */
-#define TSIG_BADTIME                            DNS_ERROR_CODE(18)      /* TSIG timestamp outisde of the time window    */
-#define TSIG_BADMODE                            DNS_ERROR_CODE(19)
-#define TSIG_BADNAME                            DNS_ERROR_CODE(20)
-#define TSIG_BADALG                             DNS_ERROR_CODE(21)
-#define TSIG_BADTRUNC                           DNS_ERROR_CODE(22)
 #define UNPROCESSABLE_MESSAGE                   DNS_ERROR_CODE(23)
 #define INVALID_PROTOCOL                        DNS_ERROR_CODE(24)
 #define INVALID_RECORD                          DNS_ERROR_CODE(25)
@@ -298,7 +301,7 @@ bool dnscore_monitored_fail(ya_result ret);
 #define RRSIG_UNSUPPORTED_COVERED_TYPE          DNS_ERROR_CODE(52)
 #define RRSIG_VERIFICATION_FAILED               DNS_ERROR_CODE(53)
 
-#define UNKNOWN_DNSSEC_ALGO                     DNS_ERROR_CODE(100)
+#define DNSSEC_ALGORITHM_UNKOWN                     DNS_ERROR_CODE(100)
 
 /// @note EAI error codes are used for getaddrinfo
 ///
@@ -349,7 +352,7 @@ void dief(ya_result error_code, const char *format, ...);
 
 void error_unregister_all();
 
-void error_register(ya_result code, const char *text);
+void error_register(ya_result code, const char * const text);
 
 /**
  * @brief Returns the string associated to an error code
@@ -371,6 +374,8 @@ void error_writetext(struct output_stream *os, ya_result code);
 
 void dnscore_register_errors();
 
+ya_result ya_ssl_error();
+
 #define DIE(code) dief((code), "%s:%i\n", __FILE__, __LINE__);abort()
 #define DIE_MSG(msg) dief(ERROR, "%s:%i %s\n", __FILE__, __LINE__, (msg));abort()
 
@@ -378,5 +383,3 @@ void dnscore_register_errors();
 
 /** @} */
 
-/*----------------------------------------------------------------------------*/
-
