@@ -185,7 +185,7 @@ message_print_format_dig_buffer(output_stream *os_, const u8 *buffer, u32 length
                 u64 next = counters.write_count + 24 + 8;
 
                 /* write NAME + alignment for next item */
-                osformat(os, ";%{dnsname}", record_wire, ' ' );
+                osformat(os, ";%{dnsname}", record_wire);
                 while(counters.write_count < next)
                 {
                     output_stream_write_u8(os, (u8)' ');
@@ -241,7 +241,7 @@ message_print_format_dig_buffer(output_stream *os_, const u8 *buffer, u32 length
                 {
                     continue;
                 }
-                else if(rtype == TYPE_OPT)
+                else if(rtype == TYPE_TSIG)
                 {
                     continue;
                 }
