@@ -297,7 +297,7 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
                 {
                     data->eos = TRUE;
 
-                    return_value = INVALID_STATE_ERROR; // OWNER OF SOA RECORD SHOULD BE ORIGIN (protocol error)
+                    return_value = MAKE_DNSMSG_ERROR(FP_XFR_QUERYERROR); // OWNER OF SOA RECORD SHOULD BE ORIGIN (protocol error)
 
                     return return_value;
                 }
@@ -577,12 +577,15 @@ xfr_input_stream_fill(input_stream *is, u32 len)
         
         while(n > 0)
         {
-            if(FAIL(ret = packet_reader_skip_fqdn(pr)))
+            if(FAIL(ret = packet_reader_skip_fqdn(pr))) // this is the domain already used for this query
             {
                 break;
             }
 
-            packet_reader_skip(pr, 4);
+            if(FAIL(ret = packet_reader_skip(pr, 4)))
+            {
+                break;
+            }
 
             n--;
         }
@@ -915,8 +918,19 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
         return return_value;
     }
 
+    if(message_get_rcode(mesg) != RCODE_NOERROR)
+    {
+        pool_release(&xfr_pool, pool);
+        return MAKE_DNSMSG_ERROR(message_get_rcode(mesg));
+    }
+
     packet_reader_init_from_message(&pr, mesg);
-    packet_reader_read_fqdn(&pr, record, RDATA_MAX_LENGTH + 1);
+
+    if(FAIL(packet_reader_read_fqdn(&pr, record, RDATA_MAX_LENGTH + 1)))
+    {
+        pool_release(&xfr_pool, pool);
+        return INVALID_PROTOCOL;
+    }
 
     if(!dnsname_equals(record, origin))
     {
