@@ -95,7 +95,8 @@
 #define MODULE_MSG_HANDLE g_dnssec_logger
 extern logger_handle *g_dnssec_logger;
 
-
+#define N3IRRVDT_TAG 0x544456525249334e
+#define RRVDATA_TAG 0x41544144565252
 
 /**
  * used by nsec3_label_link
@@ -1335,27 +1336,35 @@ nsec3_zone_set_status(zdb_zone *zone, u8 secondary_lock, u8 algorithm, u8 optout
     dynupdate_message_set_reader(&dmsg, &reader);
     u16 count = dynupdate_message_get_count(&dmsg);
 
-    packet_reader_skip(&reader, DNS_HEADER_LENGTH);
-    packet_reader_skip_fqdn(&reader);
-    packet_reader_skip(&reader, 4);
-#if ZDB_HAS_DNSSEC_SUPPORT && ZDB_HAS_RRSIG_MANAGEMENT_SUPPORT
-    if(zone_get_maintain_mode(zone) == 0)
-    {
-        zone_set_maintain_mode(zone, ZDB_ZONE_MAINTAIN_NSEC3_OPTOUT);
-    }
-#endif
+    packet_reader_skip(&reader, DNS_HEADER_LENGTH); // checked below
+    packet_reader_skip_fqdn(&reader);               // checked below
+    packet_reader_skip(&reader, 4);             // checked below
 
     ya_result ret;
-    
-    ret = dynupdate_diff(zone, &reader, count, secondary_lock, DYNUPDATE_DIFF_RUN);
-    
-    dynupdate_message_finalize(&dmsg);
-    
-    if(ret == ZDB_JOURNAL_MUST_SAFEGUARD_CONTINUITY)
+
+    if(!packet_reader_eof(&reader))
     {
-        // trigger a background store of the zone
-        
-        zdb_zone_info_background_store_zone(zone->origin);
+#if ZDB_HAS_DNSSEC_SUPPORT && ZDB_HAS_RRSIG_MANAGEMENT_SUPPORT
+        if(zone_get_maintain_mode(zone) == 0)
+        {
+            zone_set_maintain_mode(zone, ZDB_ZONE_MAINTAIN_NSEC3_OPTOUT);
+        }
+#endif
+
+        ret = dynupdate_diff(zone, &reader, count, secondary_lock, DYNUPDATE_DIFF_RUN);
+
+        dynupdate_message_finalize(&dmsg);
+
+        if(ret == ZDB_JOURNAL_MUST_SAFEGUARD_CONTINUITY)
+        {
+            // trigger a background store of the zone
+
+            zdb_zone_info_background_store_zone(zone->origin);
+        }
+    }
+    else
+    {
+        ret = MAKE_DNSMSG_ERROR(RCODE_FORMERR);
     }
         
     return ret;
@@ -1528,7 +1537,7 @@ nsec3_item_resource_record_view_data_item_set(nsec3_item_rrv_data_t *rrv_data, n
     {
         free(rrv_data->rdata);
         rrv_data->rdata_buffer_size = (required_size + 128) & ~127;
-        MALLOC_OBJECT_ARRAY_OR_DIE(rrv_data->rdata, u8, rrv_data->rdata_buffer_size, GENERIC_TAG);
+        MALLOC_OBJECT_ARRAY_OR_DIE(rrv_data->rdata, u8, rrv_data->rdata_buffer_size, RRVDATA_TAG);
     }
 
     rrv_data->rdata_size = nsec3_zone_item_to_rdata(rrv_data->n3, item, rrv_data->rdata, rrv_data->rdata_buffer_size);
@@ -1591,6 +1600,7 @@ nsec3_item_rrv_new_instance(void *data, const u8 *fqdn, u16 rtype, u16 rclass, s
 {
     (void)data;
     (void)fqdn;
+    (void)rtype;
     (void)rclass;
     yassert(rtype == TYPE_RRSIG);
     struct zdb_packed_ttlrdata *ttlrdata;
@@ -1612,7 +1622,7 @@ static const struct resource_record_view_vtbl nsec3_item_rrv_vtbl =
 void
 nsec3_item_resource_record_view_init(resource_record_view *rrv)
 {
-    ZALLOC_OBJECT_OR_DIE(rrv->data, nsec3_item_rrv_data_t, GENERIC_TAG);
+    ZALLOC_OBJECT_OR_DIE(rrv->data, nsec3_item_rrv_data_t, N3IRRVDT_TAG);
     ZEROMEMORY(rrv->data, sizeof(nsec3_item_rrv_data_t));
     rrv->vtbl = &nsec3_item_rrv_vtbl;
 }
