@@ -64,7 +64,9 @@
 #include <dnscore/serial.h>
 #include <dnscore/fdtools.h>
 #include <dnscore/tcp_io_stream.h>
+#if DNSCORE_HAS_TCP_MANAGER
 #include <dnscore/tcp_manager.h>
+#endif
 
 #include "dnsdb/zdb_types.h"
 #include "dnsdb/zdb-zone-arc.h"
@@ -618,7 +620,7 @@ zdb_zone_answer_axfr_thread(void* data_)
                 
                 zdb_zone_unlock(data_zone, ZDB_ZONE_MUTEX_SIMPLEREADER); // RC decremented
                 
-                log_err("zone write axfr: %{dnsname}: file create error for '%s': %r", data_zone_origin, buffer, serial, ret);
+                log_err("zone write axfr: %{dnsname}: file create error for '%s': %r", data_zone_origin, buffer, ret);
                 
                 data->return_code = ret;
 
@@ -710,6 +712,9 @@ zdb_zone_answer_axfr_thread(void* data_)
             
             zdb_zone_unlock(data_zone, ZDB_ZONE_MUTEX_SIMPLEREADER);
             data_zone = NULL;
+#if DNSCORE_HAS_TCP_MANAGER
+            sctx = tcp_manager_context_acquire(sctx); // because it will be release zdb_zone_answer_axfr_thread_exit
+#endif
             zdb_zone_answer_axfr_thread_exit(data);
             data = NULL; // This ensures a crash if data is used
             break;
@@ -742,7 +747,10 @@ zdb_zone_answer_axfr_thread(void* data_)
                     data->return_code = SUCCESS;
 
                     log_info("zone write axfr: %{dnsname}: releasing implicit write lock, serial is %d", data_zone_origin, serial);
-                    zdb_zone_acquire(data_zone);            
+                    zdb_zone_acquire(data_zone);
+#if DNSCORE_HAS_TCP_MANAGER
+                    sctx = tcp_manager_context_acquire(sctx); // because it will be release zdb_zone_answer_axfr_thread_exit
+#endif
                     zdb_zone_answer_axfr_thread_exit(data); // WARNING: From this point forward, 'data' cannot be used anymore 
                     data = NULL;                            //          This ensures a crash if data is used
                     zdb_zone_release_unlock(data_zone, ZDB_ZONE_MUTEX_SIMPLEREADER);
@@ -778,7 +786,10 @@ zdb_zone_answer_axfr_thread(void* data_)
                 data->return_code = SUCCESS;
 
                 log_info("zone write axfr: %{dnsname}: releasing implicit write lock, serial is %d", data_zone_origin, serial);
-                zdb_zone_acquire(data_zone);            
+                zdb_zone_acquire(data_zone);
+#if DNSCORE_HAS_TCP_MANAGER
+                sctx = tcp_manager_context_acquire(sctx); // because it will be release zdb_zone_answer_axfr_thread_exit
+#endif
                 zdb_zone_answer_axfr_thread_exit(data); // WARNING: From this point forward, 'data' cannot be used anymore 
                 data = NULL;                            //          This ensures a crash if data is used
                 zdb_zone_release_unlock(data_zone, ZDB_ZONE_MUTEX_SIMPLEREADER);
@@ -892,7 +903,7 @@ zdb_zone_answer_axfr_thread(void* data_)
 
         if(dnscore_shuttingdown())
         {
-            log_err("zone write axfr: %{dnsname}: stopping transfer to %{sockaddr} because of application shutdown", data_zone_origin, message_get_sender_sa(mesg));
+            log_info("zone write axfr: %{dnsname}: stopping transfer to %{sockaddr} because of application shutdown", data_zone_origin, message_get_sender_sa(mesg));
             break;
         }
 
@@ -1140,8 +1151,7 @@ zdb_zone_answer_axfr_thread(void* data_)
                     }
                     if(len + 2 != rdata_len)
                     {
-                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, n,
-                                rdata_len, len + 2);
+                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, rdata_len, len + 2);
                         
                         /*
                          * GOTO !!! (thread carefully)
@@ -1192,8 +1202,7 @@ zdb_zone_answer_axfr_thread(void* data_)
                     }
                     if(len != rdata_len)
                     {
-                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, n,
-                                rdata_len, len);
+                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, rdata_len, len);
                         
                         /*
                          * GOTO !!! (thread carefully)
@@ -1252,8 +1261,7 @@ zdb_zone_answer_axfr_thread(void* data_)
 
                     if(mlen + rlen + 20 != rdata_len)
                     {
-                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, n,
-                                rdata_len, mlen + rlen + 20);
+                        log_err("zone write axfr: %{dnsname}: expected rdata of %i bytes but got %i", data_zone_origin, rdata_len, mlen + rlen + 20);
                         
                         /*
                          * GOTO !!! (thread carefully)
@@ -1379,13 +1387,19 @@ zdb_zone_answer_axfr_thread(void* data_)
 
 scheduler_queue_zone_write_axfr_thread_exit:
 
+#if DNSCORE_HAS_TCP_MANAGER
+    if(sctx != NULL)
+    {
+        tcp_manager_context_release(sctx);
+    }
+#endif
+
     log_info("zone write axfr: %{dnsname}: closing file, %llu bytes sent to %{sockaddr}", data_zone_origin, total_bytes_sent, message_get_sender_sa(mesg));
 
 #if DNSCORE_HAS_TCP_MANAGER
 #if DEBUG
     log_debug("zone write axfr: %{dnsname}: closing socket %i", data_zone_origin, tcp_manager_socket(sctx));
 #endif
-    tcp_manager_close(sctx);
 #else
 #if DEBUG
     log_debug("zone write axfr: %{dnsname}: closing socket %i", data_zone_origin, tcpfd);
@@ -1397,7 +1411,10 @@ scheduler_queue_zone_write_axfr_thread_exit:
     close_ex(tcpfd);
 #endif
 #endif
-    input_stream_close(&fis);
+    if(input_stream_valid(&fis))
+    {
+        input_stream_close(&fis);
+    }
 
     message_free(mesg);
 
@@ -1418,10 +1435,16 @@ zdb_zone_answer_axfr(zdb_zone *zone, message_data *mesg, int sockfd, struct thre
     
     if(message_get_size(mesg) >= max_packet_size)
     {
-        log_err("zone write axfr: %{dnsname}: received message is already bigger than maximum message size in answer: cancelled",
-                zone->origin);
+        log_err("zone write axfr: %{dnsname}: received message is already bigger than maximum message size in answer: cancelled", zone->origin);
         return;
     }
+
+    message_data *clone = message_dup(mesg);
+    if(clone == NULL)
+    {
+        log_err("zone write axfr: %{dnsname}: received message : cancelled", zone->origin);
+        return; // BUFFER_WOULD_OVERFLOW;
+    }
         
     MALLOC_OBJECT_OR_DIE(args, zdb_zone_answer_axfr_thread_args, SHDQZWAA_TAG);
 
@@ -1442,7 +1465,7 @@ zdb_zone_answer_axfr(zdb_zone *zone, message_data *mesg, int sockfd, struct thre
 #else
     args->sockfd = sockfd;
 #endif
-    args->mesg = message_dup(mesg);
+    args->mesg = clone;
     args->packet_size_limit = max_packet_size;
     args->packet_records_limit = max_record_by_packet;
     args->compress_dname_rdata = compress_packets;
