@@ -60,7 +60,9 @@
 #include <dnscore/packet_reader.h>
 #include <dnscore/rfc.h>
 #include <dnscore/serial.h>
+#if DNSCORE_HAS_TCP_MANAGER
 #include <dnscore/tcp_manager.h>
+#endif
 
 #if DEBUG
 #include <dnscore/logger-output-stream.h>
@@ -129,11 +131,19 @@ struct zdb_zone_answer_ixfr_args
 };
 
 static void
-zdb_zone_answer_ixfr_thread_exit(zdb_zone_answer_ixfr_args* data)
+zdb_zone_answer_ixfr_thread_finalize(zdb_zone_answer_ixfr_args* data)
 {
     log_debug("zone write ixfr: ended with: %r", data->return_code);
 
     zdb_zone_release(data->zone);
+
+#if DNSCORE_HAS_TCP_MANAGER
+    if(data->sctx != NULL)
+    {
+        tcp_manager_context_release(data->sctx);
+        data->sctx = NULL;
+    }
+#endif
     
     if(data->mesg != NULL)
     {
@@ -275,6 +285,74 @@ zdb_zone_answer_ixfr_send_message(output_stream *tcpos, packet_writer *pw, messa
     return return_code;
 }
 
+static void
+zdb_zone_answer_ixfr_thread_close_finalize(zdb_zone_answer_ixfr_args* data, ya_result ret)
+{
+#if DNSCORE_HAS_TCP_MANAGER
+    tcp_manager_close(data->sctx);
+#else
+    shutdown(data->sockfd, SHUT_RDWR);
+    close_ex(data->sockfd);
+    data->sockfd = -1;
+#endif
+    data->return_code = ret;
+    zdb_zone_answer_ixfr_thread_finalize(data);
+}
+
+static ya_result
+zdb_zone_answer_ixfr_thread_read_SOA_serial(zdb_zone_answer_ixfr_args* data, packet_unpack_reader_data *purd, u32 *serialp)
+{
+    struct type_class_ttl_rdlen tctr;
+    u8 fqdn[MAX_DOMAIN_LENGTH];
+
+    if((data == NULL) || (purd == NULL) || (serialp == NULL))
+    {
+        return UNEXPECTED_NULL_ARGUMENT_ERROR;
+    }
+
+    if(FAIL(packet_reader_read_fqdn(purd, fqdn, sizeof(fqdn))))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(!dnsname_equals_ignorecase(fqdn, data->zone->origin))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_read(purd, &tctr, 10)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(tctr.qtype != TYPE_SOA)
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_skip_fqdn(purd)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_skip_fqdn(purd)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(packet_reader_available(purd) != 20)
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_read(purd, (u8*)serialp, 4)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    return SUCCESS;
+}
+
 /*
  * writes the filtered stream to a file, then adds it to the journal
  * the journal needs to give fast access to the last SOA in it ...
@@ -369,7 +447,7 @@ zdb_zone_answer_ixfr_thread(void* data_)
 
         data->return_code = MAKE_ERRNO_ERROR(ENOTSOCK);
 
-        zdb_zone_answer_ixfr_thread_exit(data);
+        zdb_zone_answer_ixfr_thread_finalize(data);
         
         return NULL;
     }
@@ -411,8 +489,8 @@ zdb_zone_answer_ixfr_thread(void* data_)
         data->return_code = ZDB_ERROR_NOSOAATAPEX;
 
         log_crit("zone write ixfr: %{dnsname}: no SOA in zone", data->zone->origin); /* will ultimately lead to the end of the program */
-        
-        zdb_zone_answer_ixfr_thread_exit(data);
+
+        zdb_zone_answer_ixfr_thread_finalize(data);
                 
         return NULL;        
     }
@@ -440,79 +518,116 @@ zdb_zone_answer_ixfr_thread(void* data_)
 
     /* Keep only the query */
 
-    packet_reader_skip_fqdn(&purd);
-    purd.offset += 4;
+    if(FAIL( packet_reader_skip_fqdn(&purd)))
+    {
+        log_crit("zone write ixfr: %{dnsname}: format error", data->zone->origin); /* will ultimately lead to the end of the program */
+        zdb_zone_answer_ixfr_thread_close_finalize(data, MAKE_DNSMSG_ERROR(RCODE_FORMERR));
+        return NULL;
+    }
 
-    message_set_size(mesg, purd.offset);
+    if(packet_reader_available(&purd) < 4 + 10)
+    {
+        log_crit("zone write ixfr: %{dnsname}: format error", data->zone->origin); /* will ultimately lead to the end of the program */
+        zdb_zone_answer_ixfr_thread_close_finalize(data, MAKE_DNSMSG_ERROR(RCODE_FORMERR));
+        return NULL;
+    }
 
-    /* Get the queried serial */
+    purd.offset += 4; // type & class
 
-    packet_reader_skip_fqdn(&purd);
-    
-    purd.offset += 2 + 2 + 4 + 2;
+    message_set_size(mesg, purd.offset); // the part after this will be overwritten later
+
+    /* Get the queried serial from the expected SOA record */
+
+    if(FAIL(zdb_zone_answer_ixfr_thread_read_SOA_serial(data, &purd, &serial)))
+    {
+        log_crit("zone write ixfr: %{dnsname}: format error", data->zone->origin); /* will ultimately lead to the end of the program */
+        zdb_zone_answer_ixfr_thread_close_finalize(data, MAKE_DNSMSG_ERROR(RCODE_FORMERR));
+        return NULL;
+    }
 
-    packet_reader_skip_fqdn(&purd);
-    packet_reader_skip_fqdn(&purd);
-    packet_reader_read(&purd, (u8*)&serial, 4);
     serial = ntohl(serial);
     
     log_debug("zone write ixfr: %{dnsname}: %{sockaddr}: client requested changes from serial %08x (%d)", data->zone->origin, message_get_sender_sa(mesg), serial, serial);
 
     message_set_authoritative_answer(mesg);
     message_set_authority_count(mesg, 0);
-    
+
+    u32 journal_serial_from, journal_serial_to;
+    return_value = zdb_zone_journal_get_serial_range(data->zone, &journal_serial_from, &journal_serial_to);
+
     dns_resource_record rr;
     dns_resource_record_init(&rr);
-    
-    if(FAIL(return_value = zdb_zone_journal_get_ixfr_stream_at_serial(data->zone, serial, &fis, &rr)))
+
+    if(ISOK(return_value))
     {
-        if(return_value == ZDB_JOURNAL_SERIAL_OUT_OF_KNOWN_RANGE)
+        if(serial_ge(serial, journal_serial_from) && serial_le(serial, journal_serial_to))
         {
-            u32 from, to;
-            
-            ya_result range_ret = zdb_zone_journal_get_serial_range(data->zone, &from, &to);
-            
-            if(ISOK(range_ret))
-            {
-                log_info("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d out of the journal range [%d; %d]", data->zone->origin, message_get_sender_sa(mesg), serial, from, to);
-            }
-            else
-            {
-                log_err("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d, but the journal range cannot be retrieved: %r", data->zone->origin, message_get_sender_sa(mesg), serial, range_ret);
-            }
+            // good
+            log_info("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d which is in [%d; %d]", data->zone->origin, message_get_sender_sa(mesg), serial, journal_serial_from, journal_serial_to);
+            return_value = zdb_zone_journal_get_ixfr_stream_at_serial(data->zone, serial, &fis, &rr);
         }
         else
         {
-            if(return_value != ZDB_ERROR_ICMTL_NOTFOUND)
+            log_notice("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d which is out of [%d; %d]", data->zone->origin, message_get_sender_sa(mesg), serial, journal_serial_from, journal_serial_to);
+            return_value = ZDB_JOURNAL_SERIAL_OUT_OF_KNOWN_RANGE;
+        }
+    }
+    else
+    {
+        if(return_value == ZDB_ERROR_ICMTL_NOTFOUND)
+        {
+            log_notice("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d but there is no journal to be found", data->zone->origin, message_get_sender_sa(mesg), serial);
+        }
+        else
+        {
+            if(return_value == ERROR)
             {
-                if(return_value != /**/ ERROR)
-                {
-                    log_err("zone write ixfr: %{dnsname}: %{sockaddr}: unable to open journal: %r", data->zone->origin, message_get_sender_sa(mesg), return_value);
-                }
-                else // a generic error occurs when the journal is being maintained
-                {
-                    log_info("zone write ixfr: %{dnsname}: %{sockaddr}: journal is busy", data->zone->origin, message_get_sender_sa(mesg));
-                    return_value = ZDB_JOURNAL_IS_BUSY;
-                }
+                return_value = ZDB_JOURNAL_IS_BUSY;
             }
-            else
+
+            if(return_value == ZDB_JOURNAL_IS_BUSY)
             {
-                log_debug("zone write ixfr: %{dnsname}: %{sockaddr}: there is no journal", data->zone->origin, message_get_sender_sa(mesg));
+                log_notice("zone write ixfr: %{dnsname}: %{sockaddr}: host asked for serial %d but the journal is being maintained", data->zone->origin, message_get_sender_sa(mesg), serial);
             }
         }
-        
+    }
+
+    if(FAIL(return_value))
+    {
         dns_resource_record_clear(&rr);
 
+        if(return_value != ZDB_JOURNAL_IS_BUSY)
+        {
 #if DNSCORE_HAS_TCP_MANAGER
-        zdb_zone_answer_axfr(data->zone, mesg, data->sctx, NULL, data->disk_tp, data->packet_size_limit, data->packet_records_limit, data->compress_dname_rdata);
-        data->sctx = NULL;
+            zdb_zone_answer_axfr(data->zone, mesg, data->sctx, NULL, data->disk_tp, data->packet_size_limit, data->packet_records_limit, data->compress_dname_rdata);
 #else
-        zdb_zone_answer_axfr(data->zone, mesg, data->sockfd, NULL, data->disk_tp, data->packet_size_limit, data->packet_records_limit, data->compress_dname_rdata);
-        data->sockfd = -1;
+            zdb_zone_answer_axfr(data->zone, mesg, data->sockfd, NULL, data->disk_tp, data->packet_size_limit, data->packet_records_limit, data->compress_dname_rdata);
+            data->sockfd = -1;
 #endif
-        data->return_code = return_value;
+        }
+        else
+        {
+#if DNSCORE_HAS_TCP_MANAGER
+            output_stream_flush(&tcpos);
+            output_stream *tcpos_filtered = buffer_output_stream_get_filtered(&tcpos);
+            fd_output_stream_detach(tcpos_filtered);
+#endif
+            message_set_status(mesg, RCODE_SERVFAIL);
+            message_transform_to_signed_error(mesg);
 
-        zdb_zone_answer_ixfr_thread_exit(data);
+#if DNSCORE_HAS_TCP_MANAGER
+            ya_result ret = message_send_tcp(mesg, tcp_manager_socket(data->sctx));
+            if(ISOK(ret))
+            {
+                tcp_manager_write_update(data->sctx, ret);
+            }
+#else
+            message_send_tcp(mesg, data->sockfd);
+#endif
+        }
+
+        data->return_code = return_value;
+        zdb_zone_answer_ixfr_thread_finalize(data);
         
         return NULL;
     }
@@ -542,8 +657,8 @@ zdb_zone_answer_ixfr_thread(void* data_)
 #endif
 
         data->return_code = BUFFER_WOULD_OVERFLOW;
-        
-        zdb_zone_answer_ixfr_thread_exit(data);
+
+        zdb_zone_answer_ixfr_thread_finalize(data);
         
         return NULL;
     }
@@ -579,8 +694,7 @@ zdb_zone_answer_ixfr_thread(void* data_)
     message_reset_buffer_size(mesg);
 
 #if DNSCORE_HAS_TCP_MANAGER
-    tcp_manager_socket_context_t *sctx = data->sctx;
-    data->sctx = NULL;
+    tcp_manager_socket_context_t *sctx = tcp_manager_context_acquire(data->sctx);
 #else
     int tcpfd = data->sockfd;
     data->sockfd = -1;
@@ -594,8 +708,8 @@ zdb_zone_answer_ixfr_thread(void* data_)
 
     data->mesg = NULL; // still need the message.  do not destroy it
     data->return_code = SUCCESS;
-    
-    zdb_zone_answer_ixfr_thread_exit(data);
+
+    zdb_zone_answer_ixfr_thread_finalize(data);
 
     /* WARNING: From this point forward, 'data' cannot be used anymore */
 
@@ -649,6 +763,10 @@ zdb_zone_answer_ixfr_thread(void* data_)
             break;
         }
 
+#if DNSCORE_HAS_TCP_MANAGER
+        tcp_manager_read_update(sctx, return_value);
+#endif
+
         // at this point, record_length >= 0
         // if record_length > 0 then tctrl has been set
         
@@ -683,9 +801,7 @@ zdb_zone_answer_ixfr_thread(void* data_)
                 }
             }
         }
-        
 
-        
         if(record_length == 0)
         {
 #if DEBUG
@@ -725,7 +841,7 @@ zdb_zone_answer_ixfr_thread(void* data_)
             // this is technically possible with an RDATA of 64K
             log_err("zone write ixfr: %{dnsname}: %{sockaddr}: ignoring record of size %u", origin, message_get_sender_sa(mesg), record_length);
             rdata_desc rr_desc = {tctrl.qtype, rdata_size, rdata_buffer};                            
-            log_err("zone write ixfr: %{dnsname}: %{sockaddr}: record is: %{dnsname} %{typerdatadesc}", origin, message_get_sender_sa(mesg), return_value, fqdn, &rr_desc);
+            log_err("zone write ixfr: %{dnsname}: %{sockaddr}: record is: %{dnsname} %{typerdatadesc}", origin, message_get_sender_sa(mesg), fqdn, &rr_desc);
             continue;
         }
         
@@ -744,6 +860,9 @@ zdb_zone_answer_ixfr_thread(void* data_)
             if(ISOK(return_value = zdb_zone_answer_ixfr_send_message(&tcpos, &pw, mesg)))
 #endif
             {
+#if DNSCORE_HAS_TCP_MANAGER
+                tcp_manager_write_update(sctx, return_value);
+#endif
                 ++pages_sent;
                 stream_serial = current_to_serial;
             }
@@ -806,8 +925,12 @@ zdb_zone_answer_ixfr_thread(void* data_)
         }
     }
 
+#if DNSCORE_HAS_TCP_MANAGER
+    output_stream_flush(&tcpos);
+    output_stream *tcpos_filtered = buffer_output_stream_get_filtered(&tcpos);
+    fd_output_stream_detach(tcpos_filtered);
+#endif
     output_stream_close(&tcpos);
-
     if(input_stream_valid(&fis))
     {
         input_stream_close(&fis);
@@ -851,14 +974,24 @@ zdb_zone_answer_ixfr(
     bool compress_dname_rdata)
 {
     zdb_zone_answer_ixfr_args* args;
-        
+
+    message_data *clone = message_dup(mesg);
+    if(clone == NULL)
+    {
+        log_warn("zone write axfr: %{dnsname}: %{sockaddr}: cannot answer, message cannot be processed", zone->origin, message_get_sender_sa(mesg));
+#if !DNSCORE_HAS_TCP_MANAGER
+        close_ex(sockfd);
+#endif
+        return; // BUFFER_WOULD_OVERFLOW;
+    }
+
     log_info("zone write ixfr: %{dnsname}: %{sockaddr}: queueing answer", zone->origin, message_get_sender_sa(mesg));
     
     MALLOC_OBJECT_OR_DIE(args, zdb_zone_answer_ixfr_args, ZAIXFRA_TAG);
     zdb_zone_acquire(zone);
     args->zone = zone;
 
-    args->mesg = message_dup(mesg);
+    args->mesg = clone;
     args->disk_tp = disk_tp;
 #if DNSCORE_HAS_TCP_MANAGER
     args->sctx = sctx;
