@@ -204,7 +204,8 @@ database_zone_path_provider(const u8* domain_fqdn, char *path_buffer, u32 path_b
         if((zone_desc->file_name == NULL) || (g_config->data_path == NULL))
         {
             zone_release(zone_desc);
-            return ZRE_NO_VALID_FILE_FOUND;
+            log_err("database: path provider: zone file name or data path not set");
+            return INVALID_STATE_ERROR;
         }
 
         const char *base_data_path = (zone_desc->file_name[0] != '/')?g_config->data_path:"";
@@ -275,7 +276,8 @@ database_zone_path_provider(const u8* domain_fqdn, char *path_buffer, u32 path_b
                 }
                 else
                 {
-                    ret = ZRE_NO_VALID_FILE_FOUND;
+                    log_err("database: path provider: transfer base path not set");
+                    ret = INVALID_STATE_ERROR;
                 }
                 
                 break;
@@ -372,7 +374,6 @@ database_info_provider(const u8 *origin, zdb_zone_info_provider_data *data, u32
                 yassert(zone_desc->journal_size_kb <= 8388608);
                 u64 max_size = zone_desc->journal_size_kb;
 
-
                 if(max_size > 0)
                 {
                     // the size has been set by the admin
@@ -646,15 +647,23 @@ database_add_key_parameters_from_zone(zdb_zone *zone, ptr_vector *keys)
  * Adds key parameters from keys removed by the message to a vector, removes said keys from the "added" vector.
  */
 
-void
+ya_result
 database_add_key_parameters_from_message(zdb_zone *zone, message_data *mesg, ptr_vector *keys, ptr_vector *removed_keys)
 {
     packet_unpack_reader_data pr;
     ya_result ret = SUCCESS;
 
     packet_reader_init_from_message(&pr, mesg);
-    packet_reader_skip_section(&pr, 0);
-    packet_reader_skip_section(&pr, 1);
+
+    if(FAIL(packet_reader_skip_section(&pr, 0)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_skip_section(&pr, 1)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
 
     // scan for added DNSKEY
 
@@ -739,6 +748,8 @@ database_add_key_parameters_from_message(zdb_zone *zone, message_data *mesg, ptr
             break;
         }
     }
+
+    return ret;
 }
 
 /**
@@ -850,7 +861,6 @@ database_zone_ensure_private_keys(zdb_zone *zone)
         do
         {
             u16 flags = DNSKEY_FLAGS(*dnskey_rrset);
-            //u8  protocol = DNSKEY_PROTOCOL(*dnskey_rrset);
             u8  algorithm = DNSKEY_ALGORITHM(*dnskey_rrset);
             u16 tag = DNSKEY_TAG(*dnskey_rrset);                  // note: expensive
             dnssec_key *key = NULL;
@@ -936,8 +946,16 @@ database_zone_ensure_private_keys_from_message(message_data *mesg)
     bool all_keys_removed = FALSE;
 
     packet_reader_init_from_message(&pr, mesg);
-    packet_reader_skip_section(&pr, 0);
-    packet_reader_skip_section(&pr, 1);
+
+    if(FAIL(packet_reader_skip_section(&pr, 0)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
+
+    if(FAIL(packet_reader_skip_section(&pr, 1)))
+    {
+        return MAKE_DNSMSG_ERROR(RCODE_FORMERR);
+    }
 
     // scan for added DNSKEY
 
@@ -1125,7 +1143,12 @@ database_update(zdb *database, message_data *mesg)
 #endif
                                 log_debug("database: update: %{dnsname}: looking for key(s) updates from the update", zone->origin);
 
-                                database_add_key_parameters_from_message(zone, mesg, &required_key_parameters, &deleted_key_parameters);
+                                ret = database_add_key_parameters_from_message(zone, mesg, &required_key_parameters, &deleted_key_parameters);
+
+                                if(FAIL(ret))
+                                {
+                                    log_info("database: update: %{dnsname}: couldn't get key parameters from message", zone->origin);
+                                }
 #if DEBUG
                                 for(int i = 0; i <= ptr_vector_last_index(&required_key_parameters); ++i)
                                 {
@@ -1356,15 +1379,8 @@ database_update(zdb *database, message_data *mesg)
                                             
                                             zdb_zone_info_background_store_zone(zone->origin);
                                         }
-                                        
-                                        if((ret & 0xffff0000) == RCODE_ERROR_BASE)
-                                        {
-                                            message_set_status(mesg, RCODE_ERROR_GETCODE(ret));
-                                        }
-                                        else
-                                        {
-                                            message_set_status(mesg, FP_RCODE_SERVFAIL);
-                                        }
+
+                                        message_set_error_status_from_result(mesg, ret);
                                     }
                                 }
                                 else
@@ -1375,7 +1391,7 @@ database_update(zdb *database, message_data *mesg)
 
                                     log_warn("database: update: %{dnsname}: prerequisites not met", message_get_canonised_fqdn(mesg));
 
-                                    message_set_status(mesg, FP_RCODE_SERVFAIL);
+                                    message_set_error_status_from_result(mesg, ret);
                                 }
 
                                 zdb_zone_double_unlock(zone, ZDB_ZONE_MUTEX_SIMPLEREADER, ZDB_ZONE_MUTEX_DYNUPDATE);
@@ -1572,6 +1588,19 @@ database_update(zdb *database, message_data *mesg)
 ya_result
 database_shutdown(zdb *database)
 {
+    if(database == NULL)
+    {
+        return UNEXPECTED_NULL_ARGUMENT_ERROR;
+    }
+
+#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_STATISTICS_SUPPORT||DNSCORE_HAS_MMAP_DEBUG_SUPPORT
+    formatln("database_shutdown(%p) begin", database);
+    debug_stat(DEBUG_STAT_TAGS|DEBUG_STAT_MMAP);
+    zalloc_print_stats(&__termout__);
+    flushout();
+    flusherr();
+#endif
+
     database_service_stop();
     
     if(database != NULL)
@@ -1582,8 +1611,16 @@ database_shutdown(zdb *database)
     
     database_finalize();
     g_config->database = NULL;
+
+#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_STATISTICS_SUPPORT||DNSCORE_HAS_MMAP_DEBUG_SUPPORT
+    formatln("database_shutdown(%p) done", database);
+    debug_stat(DEBUG_STAT_SIZES|DEBUG_STAT_TAGS|DEBUG_STAT_DUMP|DEBUG_STAT_WALK|DEBUG_STAT_MMAP);
+    zalloc_print_stats(&__termout__);
+    flushout();
+    flusherr();
+#endif
     
-    return OK;
+    return SUCCESS;
 }
 
 /**
