@@ -78,6 +78,9 @@
 #include "server_error.h"
 
 #define NOTIFY_DETAILED_LOG 0
+#define NOTIFY_CLEANUP_DUMP 0
+
+#define NOTIFY_RECEIVE_TIMEOUT_SECONDS 2
 
 #ifndef NOTIFY_DETAILED_LOG
 #if DEBUG
@@ -316,6 +319,19 @@ notify_slaveanswer(const message_data *mesg)
 
     if(notify_service_initialised)
     {
+#if DNSCORE_HAS_TSIG_SUPPORT
+        const struct tsig_item *mesg_tsig_key = message_tsig_get_key(mesg);  // pointer to the structure used for TSIG, to be used in relevant cases
+        message_data *clone = NULL;
+        if(mesg_tsig_key != NULL)
+        {
+            clone = message_dup(mesg);
+            if(clone == NULL)
+            {
+                return; // BUFFER_WOULD_OVERFLOW;
+            }
+        }
+#endif
+
         const u8 *origin = message_get_canonised_fqdn(mesg);
         const socketaddress *sa = message_get_sender(mesg);
         u8 rcode = message_get_rcode(mesg);
@@ -325,6 +341,7 @@ notify_slaveanswer(const message_data *mesg)
 
         notifymsg->payload.answer.rcode = rcode;
         notifymsg->payload.answer.aa = aa;
+
         ZALLOC_OBJECT_OR_DIE(notifymsg->payload.answer.host, host_address, HOSTADDR_TAG);
         host_address_set_with_sockaddr(notifymsg->payload.answer.host, sa);
         
@@ -334,8 +351,8 @@ notify_slaveanswer(const message_data *mesg)
         
         if(message_tsig_get_key(mesg) != NULL)
         {
-            notifymsg->payload.answer.message = message_dup(mesg);
-            notifymsg->payload.answer.host->tsig = message_tsig_get_key(mesg);
+            notifymsg->payload.answer.message = clone;
+            notifymsg->payload.answer.host->tsig = mesg_tsig_key;
         }
         else
         {
@@ -366,25 +383,26 @@ notify_masterquery_read_soa(const u8 *origin, packet_unpack_reader_data *reader,
     
     /* read and expect an SOA */
     
-    packet_reader_read_fqdn(reader, tmp, sizeof(tmp));
-
-    if(dnsname_equals(tmp, origin))
+    if(ISOK(packet_reader_read_fqdn(reader, tmp, sizeof(tmp))))
     {
-        struct type_class_ttl_rdlen tctr;
-
-        if(packet_reader_read(reader, &tctr, 10) == 10) // exact
+        if(dnsname_equals(tmp, origin))
         {
-            if((tctr.qtype == TYPE_SOA) && (tctr.qclass == CLASS_IN))
+            struct type_class_ttl_rdlen tctr;
+
+            if(packet_reader_read(reader, &tctr, 10) == 10) // exact
             {
-                if(ISOK(return_value = packet_reader_skip_fqdn(reader)))
+                if((tctr.qtype == TYPE_SOA) && (tctr.qclass == CLASS_IN))
                 {
                     if(ISOK(return_value = packet_reader_skip_fqdn(reader)))
                     {
-                        if(packet_reader_read(reader, tmp, 4) == 4) // exact
+                        if(ISOK(return_value = packet_reader_skip_fqdn(reader)))
                         {
-                            *serial = ntohl(GET_U32_AT_P(tmp));
-                            
-                            return TRUE;
+                            if(packet_reader_read(reader, tmp, 4) == 4) // exact
+                            {
+                                *serial = ntohl(GET_U32_AT_P(tmp));
+
+                                return TRUE;
+                            }
                         }
                     }
                 }
@@ -978,7 +996,7 @@ notify_message_free(notify_message *notifymsg)
             host_address_delete(notifymsg->payload.answer.host);
             if(notifymsg->payload.answer.message != NULL)
             {
-                free(notifymsg->payload.answer.message); // message_data (free, not ZFREE)
+                message_free(notifymsg->payload.answer.message); // message_data => message_free
             }
             break;
         }
@@ -1019,7 +1037,7 @@ notify_ipv4_receiver_service(struct service_worker_s *worker)
     log_info("notify: notification service IPv4 receiver started (socket %i)", send_socket4);
 
     message_data *mesg = message_new_instance();
-    tcp_set_recvtimeout(send_socket4, 2, 0); /* half a second for UDP is a lot ... */
+    tcp_set_recvtimeout(send_socket4, NOTIFY_RECEIVE_TIMEOUT_SECONDS, 0); /* half a second for UDP is a lot ... */
 
     while(service_should_run(worker))
     {
@@ -1069,7 +1087,7 @@ notify_ipv6_receiver_service(struct service_worker_s *worker)
     log_info("notify: notification service IPv6 receiver started (socket %i)", send_socket6);
 
     message_data *mesg = message_new_instance();
-    tcp_set_recvtimeout(send_socket6, 2, 0); /* half a second for UDP is a lot ... */
+    tcp_set_recvtimeout(send_socket6, NOTIFY_RECEIVE_TIMEOUT_SECONDS, 0); /* half a second for UDP is a lot ... */
 
     while(service_should_run(worker))
     {
@@ -1159,7 +1177,9 @@ notify_service_context_manage_pending_notifications(struct notify_service_contex
     {
         /* create a list of expired message_query_summary */
 
+#if NOTIFY_CLEANUP_DUMP
         log_debug("notify: cleaning up expired notifications");
+#endif
 
         message_query_summary head;
         head.next = NULL;
@@ -1287,18 +1307,22 @@ notify_service_context_manage_pending_notifications(struct notify_service_contex
                 }
                 while(current_queries_to_clear != NULL);
             }
+#if DEBUG
             else
             {
-#if DEBUG
+#if NOTIFY_CLEANUP_DUMP
                 log_debug("notify: no queries to clear");
 #endif
             }
+#endif
 
         } // if !ptr_set_isempty(&current_queries)
 #if DEBUG
         else
         {
+#if NOTIFY_CLEANUP_DUMP
             log_debug("notify: no unanswered queries");
+#endif
         }
 #endif
     }
@@ -1307,16 +1331,21 @@ notify_service_context_manage_pending_notifications(struct notify_service_contex
     {
         if(ptr_set_isempty(&ctx->notify_queries_not_answered_yet))
         {
+#if NOTIFY_CLEANUP_DUMP
             log_debug("notify: no notification queries needs to be answered");
+#endif
         }
 
         if(tus < ctx->last_current_queries_cleanup_epoch_us)
         {
+#if NOTIFY_CLEANUP_DUMP
             float dt = (ctx->last_current_queries_cleanup_epoch_us - tus) / 1000LL;
             dt /= 1000.0f;
             log_debug("notify: still %.3fus before cleaning up times out", dt);
+#endif
         }
 
+#if NOTIFY_CLEANUP_DUMP
         if(ctx->last_current_queries_cleanup_epoch_us > 0)
         {
             log_debug("notify: no timeout to handle (expect next at %llT)", ctx->last_current_queries_cleanup_epoch_us);
@@ -1325,6 +1354,7 @@ notify_service_context_manage_pending_notifications(struct notify_service_contex
         {
             log_debug("notify: no timeout to handle");
         }
+#endif
     }
 #endif
     if((ctx->last_current_queries_cleanup_epoch_us < tus) && (ctx->last_current_queries_cleanup_epoch_us > 0))
@@ -1563,28 +1593,33 @@ notify_service_context_process_next_message(struct notify_service_context *ctx,
 
                             message_query_summary *mqs = (message_query_summary*)node->value;
 
-                            yassert(mqs != NULL);
-
-                            // verify the signature
-
-                            ya_result return_value;
-
-                            if(FAIL(return_value = tsig_verify_answer(notifymsg->payload.answer.message, mqs->mac, mqs->mac_size)))
+                            if(mqs != NULL)
                             {
-                                // if everything is good, then proceed
+                                // verify the signature
 
-                                log_notice("notify: %{dnsname}: %{hostaddr}: TSIG signature verification failed: %r",
-                                        notifymsg->origin, notifymsg->payload.answer.host, return_value);
-                                // delete notifymsg
-                                notify_message_free(notifymsg);
+                                ya_result return_value;
 
-                                break;
+                                if(FAIL(return_value = tsig_verify_answer(notifymsg->payload.answer.message, mqs->mac, mqs->mac_size)))
+                                {
+                                    // if everything is good, then proceed
+
+                                    log_notice("notify: %{dnsname}: %{hostaddr}: TSIG signature verification failed: %r",
+                                            notifymsg->origin, notifymsg->payload.answer.host, return_value);
+                                    // delete notifymsg
+                                    notify_message_free(notifymsg);
+                                    break;
+                                }
+
+                                message_free(notifymsg->payload.answer.message); // message_data => message_free
+                                notifymsg->payload.answer.message = NULL;
+                                ptr_set_delete(&ctx->notify_queries_not_answered_yet, mqs);
+                                message_query_summary_delete(mqs);
+                            }
+                            else // this should never happen
+                            {
+                                log_err("notify: %{dnsname}: %{hostaddr}: invalid internal state", notifymsg->origin, notifymsg->payload.answer.host);
+                                ptr_set_delete(&ctx->notify_queries_not_answered_yet, &tmp);
                             }
-
-                            free(notifymsg->payload.answer.message); // message_data, free, not ZFREE
-                            notifymsg->payload.answer.message = NULL;
-                            ptr_set_delete(&ctx->notify_queries_not_answered_yet, mqs);
-                            message_query_summary_delete(mqs);
                         } /* end of TSIG verification, with success*/
 #endif
                         ha = host_address_remove_host_address(&notify_zones_notifymsg->payload.notify.hosts_list, notifymsg->payload.answer.host);
@@ -1670,10 +1705,19 @@ static void
 notify_service_context_send_notifications(struct notify_service_context *ctx)
 {
 #if DEBUG
-    log_debug("notify: sending notifications");
+    if(ptr_set_isempty(&ctx->notifications_being_sent))
+    {
+        log_debug("notify: no notification to send");
+        return;
+    }
+    else
+    {
+        log_debug("notify: sending notifications");
+    }
 #endif
 
     time_t now = time(NULL);
+    int total_sent = 0;
 
     ptr_set_iterator notifications_being_sent_iter;
     ptr_set_iterator_init(&ctx->notifications_being_sent, &notifications_being_sent_iter);
@@ -1718,11 +1762,12 @@ notify_service_context_send_notifications(struct notify_service_context *ctx)
 
             ya_result ret = notify_send(ha, ctx->mesg, id, notifymsg->origin, notifymsg->payload.notify.ztype, notifymsg->payload.notify.zclass);
 
-
             host_address *ha_next = ha->next;
 
             if(ISOK(ret))
             {
+                ++total_sent;
+
                 message_query_summary* mqs;
                 ZALLOC_OBJECT_OR_DIE( mqs, message_query_summary, MSGQSUMR_TAG);
                 message_query_summary_init(mqs, id, ha, ctx->mesg);
@@ -1824,7 +1869,10 @@ notify_service_context_send_notifications(struct notify_service_context *ctx)
     ptr_vector_clear(&ctx->todelete);
 
 #if DEBUG
-    log_debug("notify: notifications sent");
+    if(total_sent > 0)
+    {
+        log_debug("notify: %i notifications sent", total_sent);
+    }
 #endif
 }
 
@@ -1973,7 +2021,9 @@ notify_service(struct service_worker_s *worker)
         s64 loop_now = loop_start;
         s64 loop_count = 0;
         bool long_accumulation = FALSE;
+#if NOTIFY_CLEANUP_DUMP
         bool no_message_in_queue = FALSE;
+#endif
         bool is_shutting_down = !service_should_run(worker);
 
         // the loop will always enter at least once once
@@ -1986,7 +2036,9 @@ notify_service(struct service_worker_s *worker)
 
             if(async == NULL)   /*if no message is in the queue, proceed to next step */
             {
+#if NOTIFY_CLEANUP_DUMP
                 no_message_in_queue = TRUE;
+#endif
                 break;
             }
             
@@ -1995,7 +2047,9 @@ notify_service(struct service_worker_s *worker)
             if(notifymsg == NULL) /*if no message is in the queue, proceed to next step (probably irrelevant) */
             {
                 async_message_release(async);
+#if NOTIFY_CLEANUP_DUMP
                 no_message_in_queue = TRUE;
+#endif
                 break;
             }
 
@@ -2030,10 +2084,12 @@ notify_service(struct service_worker_s *worker)
             log_debug("notify: notification service accumulated queries for %fms (%lli queries)", ((1.0 * (loop_now - loop_start)) / 1000.0), loop_count);
         }
 
+#if NOTIFY_CLEANUP_DUMP
         if(no_message_in_queue)
         {
             log_debug1("notify: notification service has no more messages queued");
         }
+#endif
 
         if(is_shutting_down)
         {
@@ -2174,11 +2230,21 @@ notify_slaves_alarm(void *args_, bool cancel)
 static bool
 notify_slaves_convert_domain_to_notify(notify_message *message)
 {
+    if(message->payload.type == NOTIFY_MESSAGE_TYPE_NOTIFY)
+    {
+        return TRUE;
+    }
+
     if(!notify_service_initialised)
     {
         return FALSE;
     }
 
+    if(message->payload.type != NOTIFY_MESSAGE_TYPE_DOMAIN)
+    {
+        return FALSE;
+    }
+
     /*
      * Build a list of IPs to contact
      * The master in the SOA must not be in this list
@@ -2244,12 +2310,7 @@ notify_slaves_convert_domain_to_notify(notify_message *message)
 
             u8 *soa_mname = ZDB_PACKEDRECORD_PTR_RDATAPTR(soa);
             u32 soa_mname_size = dnsname_len(soa_mname);
-            /*
-            u8 *soa_rname = soa_mname + soa_mname_size;
-            u8 *serial_ptr = soa_rname + dnsname_len(soa_rname);
-            u32 serial = *((u32*)serial_ptr);
-            serial = ntohl(serial);
-            */
+
             for(zdb_packed_ttlrdata *nsp = ns; nsp != NULL; nsp = nsp->next)
             {
                 u32 ns_dname_size = ZDB_PACKEDRECORD_PTR_RDATASIZE(nsp);
@@ -2263,7 +2324,7 @@ notify_slaves_convert_domain_to_notify(notify_message *message)
                     }
                 }
 
-                /* valid candidate : get its IP, later */
+                // valid candidate : append IP addresses
             
                 if(zdb_append_ip_records(db, ns_dname, &list) <= 0) // zone is locked
                 {
@@ -2386,8 +2447,6 @@ notify_clear(const u8 *origin)
     async_message_call(&notify_handler_queue, async);
 }
 
-
-
 ya_result
 notify_service_init()
 {
@@ -2428,7 +2487,7 @@ notify_service_init()
 
         if(ISOK(err = service_init_ex(&notify_handler, notify_service, "yadifad-notify", workers )))
         {
-            async_queue_init(&notify_handler_queue, 10000000, 1, 1, "notify");
+            async_queue_init(&notify_handler_queue, 10000000, 1, 1, "notify"); // note: it's implemented as a linked list
             
             notify_service_initialised = TRUE;
         }
@@ -2498,9 +2557,19 @@ notify_service_finalize()
     if(notify_service_initialised)
     {
         notify_service_initialised = FALSE;
+
+        if(send_socket4 >= 0)
+        {
+            shutdown(send_socket4, SHUT_RDWR);
+        }
+
+        if(send_socket6 >= 0)
+        {
+            shutdown(send_socket6, SHUT_RDWR);
+        }
         
         err = notify_service_stop();
-        
+
         service_finalize(&notify_handler);
         
         /* once the tree has been scanned, destroy every node listed */
