@@ -663,19 +663,10 @@ server_rw_process_message_udp(struct network_thread_context_s *ctx, message_data
     ya_result ret;
     if(ISOK(ret = server_process_message_udp((network_thread_context_base_t*)ctx, mesg)))
     {
-        ssize_t sent;
-
-        while((sent = message_send_udp(mesg, ctx->base.sockfd)) < 0)
+        if(ISOK(ret = message_send_udp(mesg, ctx->base.sockfd)))
         {
-            int error_code = errno;
-
-            if(error_code != EINTR)
-            {
-                return MAKE_ERRNO_ERROR(error_code);
-            }
+            ctx->statistics.udp_output_size_total += ret;
         }
-
-        ctx->statistics.udp_output_size_total += sent;
     }
 
     return ret;
@@ -802,7 +793,7 @@ server_rw_udp_sender_thread(void *parms)
                             else if(dest_port < 0)
                             {
                                 log_err("server-rw: error replying to message %04hx %{dnsname} %{dnstype} invalid IP family",
-                                        ntohs(message_get_id(mesg)), message_get_canonised_fqdn(mesg), message_get_query_type_ptr(mesg), mesg->_msghdr.msg_name);
+                                        ntohs(message_get_id(mesg)), message_get_canonised_fqdn(mesg), message_get_query_type_ptr(mesg));
                             }
                             else
                             {
@@ -910,7 +901,7 @@ server_rw_udp_sender_thread(void *parms)
 
                             if(g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE)
                             {
-                                log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_ALL);
+                                log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_BUFFER);
                             }
                         }
                     }
@@ -1038,7 +1029,7 @@ server_rw_query_loop(struct service_worker_s *worker)
      * the speed a bit.
      */
 
-    FD_ZERO(&read_set_init);    
+    FD_ZERO(&read_set_init);
     s32 reader_by_fd = g_server_context.udp_socket_count / g_server_context.udp_unit_per_interface;
     s32 cpu_count = sys_get_cpu_count();
     if(reader_by_fd > cpu_count)
@@ -1101,7 +1092,7 @@ server_rw_query_loop(struct service_worker_s *worker)
                 mutex_lock(&ctx->mtx);
                 cond_notify(&ctx->cond);
                 mutex_unlock(&ctx->mtx);
-                
+
                 break;
             }
          
@@ -1302,8 +1293,8 @@ server_rw_query_loop(struct service_worker_s *worker)
 #if HAS_ZALLOC_STATISTICS_SUPPORT
                         zalloc_print_stats(termout);
 #endif
-#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT
-                        debug_stat(DEBUG_STAT_SIZES|DEBUG_STAT_TAGS); // do NOT enable the dump
+#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_STATISTICS_SUPPORT||DNSCORE_HAS_MMAP_DEBUG_SUPPORT
+                        debug_stat(DEBUG_STAT_TAGS|DEBUG_STAT_MMAP); // do NOT enable the dump
 #endif
                         journal_log_status();
 
@@ -1358,7 +1349,7 @@ server_rw_query_loop(struct service_worker_s *worker)
     log_info("server-rw: cleaning up");
 
     thread_pool_destroy(server_udp_thread_pool);
-    //server_udp_thread_pool = NULL;
+    server_udp_thread_pool = NULL;
     
     network_thread_context_array_finalize(&ctxa);
 
