@@ -123,6 +123,7 @@ typedef cpuset_t cpu_set_t;
 
 #define SVRPLBIN_TAG 0x4e49424c50525653
 #define SVRPLBOT_TAG 0x544f424c50525653
+#define SSMPOOLB_TAG 0x424c4f4f504d5353
 
 // allow an external definition of the backlog queue size and L1 parameters
 
@@ -356,7 +357,7 @@ server_sm_udp_worker_thread(void *parms)
 
     u8 *pool_buffer;
     size_t pool_buffer_size = 65536;
-    MALLOC_OBJECT_ARRAY_OR_DIE(pool_buffer, u8, pool_buffer_size, GENERIC_TAG);
+    MALLOC_OBJECT_ARRAY_OR_DIE(pool_buffer, u8, pool_buffer_size, SSMPOOLB_TAG);
 
     message_data *mesg;
     mesg = message_new_instance();
@@ -373,13 +374,13 @@ server_sm_udp_worker_thread(void *parms)
 	    message_reset_control_size(mesg);
 	    ya_result ret = message_recv_udp(mesg, fd);
 #if DEBUG
-	     log_debug("server_sm_udp_worker_thread(%i, %i): recvmmsg: %i", ctx->base.idx, fd, ret);
+	     log_debug("server_sm_udp_worker_thread(%i, %i): recvmsg: %i", ctx->base.idx, fd, ret);
 #endif
         if(FAIL(ret))
         {
 #if DEBUG
             int err = ERRNO_ERROR;
-            log_info("message_recv_udp %i returned %i : %r", fd, ret, err);
+            log_debug("message_recv_udp %i returned %i : %r", fd, ret, err);
 #endif
             if(service_should_reconfigure_or_stop(ctx->base.worker))
             {
@@ -390,7 +391,7 @@ server_sm_udp_worker_thread(void *parms)
             continue;
         }
 #if DEBUG
-        log_info("server_sm_udp_worker_thread: received %u packets (DEBUG)", ret);
+        log_debug("server_sm_udp_worker_thread: received %u packets (DEBUG)", ret);
 #endif
         local_statistics_udp_input_count++;
 
@@ -411,7 +412,15 @@ server_sm_udp_worker_thread(void *parms)
             {
                 // that message will be replied to
                 ret = message_send_udp(mesg, fd);
-                ctx->statistics.udp_output_size_total += ret;
+
+                if(ISOK(ret))
+                {
+                    ctx->statistics.udp_output_size_total += ret;
+                }
+                else
+                {
+                    log_err("server-sm: could not reply though UDP (socket %i): %r", fd, ret);
+                }
             }
             else
             {
@@ -419,19 +428,19 @@ server_sm_udp_worker_thread(void *parms)
                 {
                     //  ignore
 #if DEBUG
-                    log_info("server_sm_udp_worker_thread: good-dropped %d bytes from %{sockaddr}", message_get_size(mesg), message_get_sender_sa(mesg));
+                    log_debug("server_sm_udp_worker_thread: good-dropped %d bytes from %{sockaddr}", message_get_size(mesg), message_get_sender_sa(mesg));
                     message_log(MODULE_MSG_HANDLE, LOG_INFO, mesg);
 #endif
                 }
                 else if(ret == STOPPED_BY_APPLICATION_SHUTDOWN)
                 {
 #if DEBUG
-                    log_info("server_sm_udp_worker_thread: STOPPED_BY_APPLICATION_SHUTDOWN ?");
+                    log_debug("server_sm_udp_worker_thread: STOPPED_BY_APPLICATION_SHUTDOWN ?");
 #endif
                     if(service_should_reconfigure_or_stop(ctx->base.worker))
                     {
 #if DEBUG
-                        log_info("server_sm_udp_worker_thread: STOPPED_BY_APPLICATION_SHUTDOWN !");
+                        log_debug("server_sm_udp_worker_thread: STOPPED_BY_APPLICATION_SHUTDOWN !");
 #endif
                         /*
                          * GOTO!
@@ -446,7 +455,7 @@ server_sm_udp_worker_thread(void *parms)
                 {
                     // something happened
 #if DEBUG
-                    log_info("server_sm_udp_worker_thread: bad-dropped %d bytes from %{sockaddr}", message_get_size(mesg), message_get_sender_sa(mesg));
+                    log_debug("server_sm_udp_worker_thread: bad-dropped %d bytes from %{sockaddr}", message_get_size(mesg), message_get_sender_sa(mesg));
                     message_log(MODULE_MSG_HANDLE, LOG_INFO, mesg);
 #endif
                 }
@@ -460,7 +469,7 @@ server_sm_udp_worker_thread(void *parms)
         else // if(dest_port < 0)
         {
             log_err("server-sm: error replying to message %04hx %{dnsname} %{dnstype} invalid IP family",
-                    ntohs(message_get_id(mesg)), message_get_canonised_fqdn(mesg), message_get_query_type_ptr(mesg), mesg->_msghdr.msg_name);
+                    ntohs(message_get_id(mesg)), message_get_canonised_fqdn(mesg), message_get_query_type_ptr(mesg));
         }
 
     } // for "ever" loop
@@ -799,8 +808,8 @@ server_sm_query_loop(struct service_worker_s *worker)
 #if HAS_ZALLOC_STATISTICS_SUPPORT
                         zalloc_print_stats(termout);
 #endif
-#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT
-                        debug_stat(DEBUG_STAT_SIZES|DEBUG_STAT_TAGS); // do NOT enable the dump
+#if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_DEBUG_SUPPORT||DNSCORE_HAS_ZALLOC_STATISTICS_SUPPORT||DNSCORE_HAS_MMAP_DEBUG_SUPPORT
+                        debug_stat(DEBUG_STAT_TAGS|DEBUG_STAT_MMAP); // do NOT enable the dump
 #endif
                         journal_log_status();
 
@@ -873,6 +882,7 @@ server_sm_query_loop(struct service_worker_s *worker)
     log_info("server-sm: cleaning up: stopping thread pool");
 
     thread_pool_destroy(server_udp_thread_pool);
+    server_udp_thread_pool = NULL;
 
     log_info("server-sm: cleaning up: releasing context");
     
