@@ -642,13 +642,37 @@ server_context_new_socket(struct addrinfo *addr, int sock_type, bool reuse_port)
         
         if(sock_type == SOCK_DGRAM)
         {
+            /*
+             * @note https://stackoverflow.com/questions/46353380/invalid-argument-error-for-sendmsg-on-some-freebsd-systems
+             * It turns out FreeBSD is very picky when it allows the use of IP_SENDSRCADDR on a UDP socket.
+             * If the socket is bound to INADDR_ANY my code works fine. If the socket is bound to a single IP,
+             * then sendmsg() returns EINVAL (invalid argument).
+             *
+             * ...
+             *
+             * Note that it's not the issue that has been reported 202102xx on GitHub.
+             */
 
-            socket_server_opensocket_setopt_ignore_error(&ctx, IPPROTO_IP, DSTADDR_SOCKOPT, &on, sizeof(on));
 #if __FreeBSD__
-            socket_server_opensocket_setopt_ignore_error(&ctx, IPPROTO_IPV6, DSTADDR_SOCKOPT, &on, sizeof(on));
+            if(host_address_is_any(addr))
+            {
+#endif
+                socket_server_opensocket_setopt_ignore_error(&ctx, IPPROTO_IP, DSTADDR_SOCKOPT, &on, sizeof(on));
+#if __FreeBSD__
+            }
+#endif
+
+            //
+#if __FreeBSD__
+            if(host_address_is_any(addr))
+            {
+                socket_server_opensocket_setopt_ignore_error(&ctx, IPPROTO_IPV6, DSTADDR_SOCKOPT, &on, sizeof(on));
+            }
 #else
             socket_server_opensocket_setopt(&ctx, IPPROTO_IPV6, DSTADDR_SOCKOPT, &on, sizeof(on));
 #endif
+            //
+
             socket_server_opensocket_setopt_ignore_result(&ctx, IPPROTO_IPV6, DSTADDR6_SOCKOPT, &on, sizeof(on));
 #if WIN32
 #else
@@ -1143,7 +1167,7 @@ server_context_create()
         {
             struct addrinfo *tcp_addr = g_server_context.tcp_interface[intf_idx];
 
-            log_debug("UDP listener socket for %{sockaddr} will be opened", tcp_addr->ai_addr);
+            log_debug("TCP listener socket for %{sockaddr} will be opened", tcp_addr->ai_addr);
 
             for(u32 n = 0; n < g_server_context.tcp_unit_per_interface; ++n)
             {
@@ -1155,12 +1179,6 @@ server_context_create()
 
                     server_context_set_socket_name(sockfd, (struct sockaddr*)tcp_addr->ai_addr);
 
-                    if(FAIL(ret = fd_setnonblocking(sockfd)))
-                    {
-                        server_context_destroy();
-                        return ret;
-                    }
-
                     /* For TCP only, listen to it... */
                     if(FAIL(listen(sockfd, g_config->tcp_queue_size)))
                     {
@@ -1199,6 +1217,12 @@ server_context_destroy()
 bool
 server_context_matches_config()
 {
+    /**
+     * If the number of addresses is the same and
+     * if all the addresses defined in the configuration are matching one address in the context
+     * then the configuration are matched.
+     */
+
     host_address *real_listen = host_address_copy_list(g_config->listen);
 
     u32 interfaces_count = host_address_count(real_listen);
@@ -1207,18 +1231,18 @@ server_context_matches_config()
         host_address_delete_list(real_listen);
         return FALSE;
     }
-    
+
     for(u32 i = 0; i < g_server_context.listen_count; ++i)
     {
         bool match = FALSE;
         host_address *ha = real_listen;
         while(ha != NULL)
         {
-            if(host_address_list_equals(g_server_context.listen[i], ha))
+            if(host_address_equals(g_server_context.listen[i], ha))
             {
                 match = TRUE;
+                break;
             }
-            
             ha = ha->next;
         }
         
