@@ -79,10 +79,10 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                 {
                     case CLASS_IN:
                     {
-                        local_statistics->udp_queries_count++;
-
                         log_query(ctx->sockfd, mesg);
 
+                        local_statistics->udp_queries_count++;
+
                         switch(message_get_query_type(mesg))
                         {
                             default:
@@ -200,36 +200,67 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
 
                         if(dnsname_verify_charspace(canonized_fqdn))
                         {
-                            log_notice("query (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
-                                       ntohs(message_get_id(mesg)),
-                                       message_get_flags_hi(mesg),message_get_flags_lo(mesg),
-                                       canonized_fqdn,
-                                       message_get_query_type_ptr(mesg),
-                                       message_get_query_class_ptr(mesg),
-                                       RCODE_ERROR_CODE(message_get_status(mesg)),
-                                       ret,
-                                       message_get_sender_sa(mesg),
-                                       message_get_size_u16(mesg),
-                                       message_tsig_get_name(mesg),
-                                       epoch,
-                                       epoch,
-                                       fudge);
+                            if(message_has_tsig(mesg))
+                            {
+                                log_notice("query (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg),
+                                           message_tsig_get_name(mesg),
+                                           epoch,
+                                           epoch,
+                                           fudge);
+                            }
+                            else
+                            {
+                                log_notice("query (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg));
+                            }
                         }
                         else
                         {
-                            log_notice("query (%04hx) [%02x|%02x] <INVALID> %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
-                                       ntohs(message_get_id(mesg)),
-                                       message_get_flags_hi(mesg),message_get_flags_lo(mesg),
-                                       message_get_query_type_ptr(mesg),
-                                       message_get_query_class_ptr(mesg),
-                                       RCODE_ERROR_CODE(message_get_status(mesg)),
-                                       ret,
-                                       message_get_sender_sa(mesg),
-                                       message_get_size_u16(mesg),
-                                       message_tsig_get_name(mesg),
-                                       epoch,
-                                       epoch,
-                                       fudge);
+                            if(message_has_tsig(mesg))
+                            {
+                                log_notice("query (%04hx) [%02x|%02x] <INVALID> %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg),
+                                           message_tsig_get_name(mesg),
+                                           epoch,
+                                           epoch,
+                                           fudge);
+                            }
+                            else
+                            {
+                                log_notice("query (%04hx) [%02x|%02x] <INVALID> %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg));
+                            }
                         }
                     }
                 }
@@ -252,7 +283,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
 
                 if((ret == UNPROCESSABLE_MESSAGE) && (g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE))
                 {
-                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_ALL);
+                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_BUFFER);
                 }
 
                 /*
@@ -264,7 +295,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                 {
                     message_edns0_clear_undefined_flags(mesg);
 
-                    if(!message_has_tsig(mesg))
+                    if(!message_has_tsig(mesg) && (message_get_status(mesg) != FP_RCODE_NOTAUTH))
                     {
                         message_transform_to_error(mesg);
                     }
@@ -369,20 +400,36 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                             s64 epoch = message_tsig_get_epoch(mesg);
                             s64 fudge = message_tsig_get_fudge(mesg);
 
-                            log_notice("notify (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
-                                       ntohs(message_get_id(mesg)),
-                                       message_get_flags_hi(mesg),message_get_flags_lo(mesg),
-                                       canonized_fqdn,
-                                       message_get_query_type_ptr(mesg),
-                                       message_get_query_class_ptr(mesg),
-                                       RCODE_ERROR_CODE(message_get_status(mesg)),
-                                       ret,
-                                       message_get_sender_sa(mesg),
-                                       message_get_size_u16(mesg),
-                                       message_tsig_get_name(mesg),
-                                       epoch,
-                                       epoch,
-                                       fudge);
+                            if(message_has_tsig(mesg))
+                            {
+                                log_notice("notify (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg),
+                                           message_tsig_get_name(mesg),
+                                           epoch,
+                                           epoch,
+                                           fudge);
+                            }
+                            else
+                            {
+                                log_notice("notify (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg));
+                            }
                         }
                     }
                     else
@@ -417,7 +464,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
 
                 if(ret == UNPROCESSABLE_MESSAGE && (g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE))
                 {
-                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_ALL);
+                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_BUFFER);
                 }
 
                 /*
@@ -429,7 +476,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                 {
                     message_edns0_clear_undefined_flags(mesg);
 
-                    if(!message_has_tsig(mesg))
+                    if(!message_has_tsig(mesg) && (message_get_status(mesg) != FP_RCODE_NOTAUTH))
                     {
                         message_transform_to_error(mesg);
                     }
@@ -461,9 +508,20 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                          */
 
                         local_statistics->udp_updates_count++;
-                        dynupdate_query_service_enqueue(database, mesg, fd);
-
-                        return SUCCESS_DROPPED; // NOT break;
+                        if(ISOK(dynupdate_query_service_enqueue(database, mesg, fd)))
+                        {
+                            return SUCCESS_DROPPED; // NOT break;
+                        }
+                        else
+                        {
+                            log_warn("update [%04hx] %{dnsname} %{dnstype} %{dnsclass} (%{sockaddrip}) : cannot enqueue the update message",
+                                     ntohs(message_get_id(mesg)),
+                                     message_get_canonised_fqdn(mesg), message_get_query_type_ptr(mesg), message_get_query_class_ptr(mesg),
+                                     message_get_sender_sa(mesg));
+                            message_make_error(mesg, FP_RCODE_SERVFAIL);
+                            local_statistics->udp_fp[FP_RCODE_SERVFAIL]++;
+                            return SUCCESS;         // needs to be >= 0: the server will send the SERVFAIL message
+                        }
 #else
                         message_make_error(mesg, FP_FEATURE_DISABLED);
                         local_statistics->udp_fp[FP_FEATURE_DISABLED]++;
@@ -509,20 +567,36 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                             s64 epoch = message_tsig_get_epoch(mesg);
                             s64 fudge = message_tsig_get_fudge(mesg);
 
-                            log_notice("update (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
-                                       ntohs(message_get_id(mesg)),
-                                       message_get_flags_hi(mesg),message_get_flags_lo(mesg),
-                                       canonized_fqdn,
-                                       message_get_query_type_ptr(mesg),
-                                       message_get_query_class_ptr(mesg),
-                                       RCODE_ERROR_CODE(message_get_status(mesg)),
-                                       ret,
-                                       message_get_sender_sa(mesg),
-                                       message_get_size_u16(mesg),
-                                       message_tsig_get_name(mesg),
-                                       epoch,
-                                       epoch,
-                                       fudge);
+                            if(message_has_tsig(mesg))
+                            {
+                                log_notice("update (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu key=%{dnsname} epoch=%lli (%T) +-%llis",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg),
+                                           message_tsig_get_name(mesg),
+                                           epoch,
+                                           epoch,
+                                           fudge);
+                            }
+                            else
+                            {
+                                log_notice("update (%04hx) [%02x|%02x] %{dnsname} %{dnstype} %{dnsclass} : %r (%r) (%{sockaddrip}) size=%hu",
+                                           ntohs(message_get_id(mesg)),
+                                           message_get_flags_hi(mesg),message_get_flags_lo(mesg),
+                                           canonized_fqdn,
+                                           message_get_query_type_ptr(mesg),
+                                           message_get_query_class_ptr(mesg),
+                                           RCODE_ERROR_CODE(message_get_status(mesg)),
+                                           ret,
+                                           message_get_sender_sa(mesg),
+                                           message_get_size_u16(mesg));
+                            }
                         }
                     }
                     else
@@ -557,7 +631,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
 
                 if((ret == UNPROCESSABLE_MESSAGE) && (g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE))
                 {
-                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_ALL);
+                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_BUFFER);
                 }
 
                 /*
@@ -569,7 +643,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                 {
                     message_edns0_clear_undefined_flags(mesg);
 
-                    if(!message_has_tsig(mesg))
+                    if(!message_has_tsig(mesg) && (message_get_status(mesg) != FP_RCODE_NOTAUTH))
                     {
                         message_transform_to_error(mesg);
                     }
@@ -593,16 +667,7 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
         }
         default:
         {
-            ret = MAKE_DNSMSG_ERROR(FP_RCODE_NOTIMP);
-            /*ret = */ message_process_query(mesg);
-            message_set_status(mesg, FP_RCODE_NOTIMP);
-
-            if(ctx->must_stop)
-            {
-                return STOPPED_BY_APPLICATION_SHUTDOWN; // shutdown
-            }
-
-            log_notice("opcode-%i (%04hx) [%02x|%02x] QC=%hu AN=%hu NS=%hu AR=%hu : %r (%r) (%{sockaddrip}) size=%hu",
+            log_notice("opcode-%i (%04hx) [%02x|%02x] QC=%hu AN=%hu NS=%hu AR=%hu (%{sockaddrip}) size=%hu",
                        (u32)(message_get_opcode(mesg) >> OPCODE_SHIFT),
                        ntohs(message_get_id(mesg)),
                        message_get_flags_hi(mesg),message_get_flags_lo(mesg),
@@ -610,22 +675,41 @@ server_process_message_udp(network_thread_context_base_t *ctx, message_data *mes
                        message_get_answer_count(mesg), // AC
                        message_get_authority_count(mesg), // NS
                        message_get_additional_count(mesg), // AR
-                       MAKE_DNSMSG_ERROR(message_get_status(mesg)),
-                       ret,
                        message_get_sender_sa(mesg),
                        message_get_size_u16(mesg));
 
+            message_process_lenient(mesg);
+
+            if(message_get_status(mesg) == RCODE_OK) // else a TSIG may have some complain
+            {
+                message_set_status(mesg, FP_RCODE_NOTIMP);
+                message_update_answer_status(mesg);
+
+#if DNSCORE_HAS_TSIG_SUPPORT
+                if(message_has_tsig(mesg))
+                {
+                    tsig_sign_answer(mesg);
+                }
+#endif
+            }
+
+            local_statistics->udp_undefined_count++;
+
+            if(ctx->must_stop)
+            {
+                return STOPPED_BY_APPLICATION_SHUTDOWN; // shutdown
+            }
+
             if(g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE)
             {
-                log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_ALL);
+                log_memdump_ex(MODULE_MSG_HANDLE, MSG_WARNING, message_get_buffer(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_BUFFER);
             }
 
             if((message_get_status(mesg) != RCODE_FORMERR) || ((g_config->server_flags & SERVER_FL_ANSWER_FORMERR) != 0))
             {
-                message_edns0_clear_undefined_flags(mesg);
-
-                if(!message_has_tsig(mesg))
+                if(!message_has_tsig(mesg) && (message_get_status(mesg) != FP_RCODE_NOTAUTH))
                 {
+                    message_edns0_clear_undefined_flags(mesg);
                     message_transform_to_error(mesg);
                 }
             }
