@@ -100,6 +100,8 @@ extern logger_handle *g_keyroll_logger;
 #define KEYROLL_CONFIG_SECTION "yakeyrolld"
 #define RELEASEDATE YADIFA_DNSCORE_RELEASE_DATE
 
+// mount -t tmpfs -o size=16384 tmpfs /registry/yadifa/var/log/yakeyrolld
+
 static random_ctx rnd;
 
 enum PROGRAM_MODE
@@ -897,6 +899,12 @@ program_mode_play(const u8 *domain, bool does_loop)
     // _ we know the present state on the server
     // _ we know the plan folder for this domain exists
 
+    if(does_loop)
+    {
+        log_info("play: %{dnsname}: loading plan", domain);
+        logger_flush();
+    }
+
     if(FAIL(ret = keyroll_plan_load(&keyroll)))
     {
         if(ret != MAKE_ERRNO_ERROR(ENOENT))
@@ -908,6 +916,8 @@ program_mode_play(const u8 *domain, bool does_loop)
             log_info("play: %{dnsname}: there are no plans on storage (%s)", domain, g_config.plan_path);
         }
 
+        logger_flush();
+
         keyroll_finalize(&keyroll);
 
         return ret;
@@ -1034,6 +1044,7 @@ program_mode_play(const u8 *domain, bool does_loop)
             if(dnscore_shuttingdown())
             {
                 log_info("play: %{dnsname}: shutting down", domain);
+                logger_flush();
                 break;
             }
 
@@ -1064,8 +1075,8 @@ program_mode_play(const u8 *domain, bool does_loop)
                     case MAKE_ERRNO_ERROR(ETIMEDOUT):
                     case MAKE_ERRNO_ERROR(EADDRNOTAVAIL):
                     case MAKE_ERRNO_ERROR(EAGAIN):
-                    case DNS_ERROR_CODE(RCODE_SERVFAIL):
-                    case DNS_ERROR_CODE(RCODE_REFUSED):
+                    case MAKE_DNSMSG_ERROR(RCODE_SERVFAIL):
+                    case MAKE_DNSMSG_ERROR(RCODE_REFUSED):
                     case UNABLE_TO_COMPLETE_FULL_READ:
                     {
                         ++consecutive_errors;
@@ -1135,7 +1146,7 @@ program_mode_play_thread(void *args_)
 
         if(ISOK(ret))
         {
-            log_warn("%{dnsname}: key roll stopped", args->fqdn);
+            log_info("%{dnsname}: key roll stopped", args->fqdn);
             break;
         }
         else
@@ -1144,9 +1155,15 @@ program_mode_play_thread(void *args_)
             {
                 log_warn("%{dnsname}: trying again from the start", args->fqdn);
             }
+            else if(ret == STOPPED_BY_APPLICATION_SHUTDOWN)
+            {
+                log_warn("%{dnsname}: keyroll is shutting down", args->fqdn);
+                break;
+            }
             else
             {
                 log_err("%{dnsname}: shutting down (%r)", args->fqdn, ret);
+                logger_flush();
                 dnscore_shutdown();
                 break;
             }
@@ -1236,7 +1253,7 @@ program_mode_play_all(bool does_loop, bool daemonise)
             args[i].fqdn = fqdn; // VS false positive (nonsense)
             args[i].does_loop = does_loop;
 
-            thread_pool_enqueue_call(tp, program_mode_play_thread, &args[i], NULL, domain);
+            thread_pool_enqueue_call(tp, program_mode_play_thread, &args[i], &counter, domain);
         }
 
         // ensure the counter was incremented
@@ -1245,16 +1262,52 @@ program_mode_play_all(bool does_loop, bool daemonise)
 
         // wait for the shutdown or for workers to stop
 
-        while(!dnscore_shuttingdown() && (thread_pool_counter_get_value(&counter) > 0))
+        for(;;)
+        {
+            ret = thread_pool_counter_wait_equal_with_timeout(&counter, 0, ONE_SECOND_US * 30);
+
+            log_debug("keyroll: waiting for the threads to stop (%r)", ret);
+
+            if(dnscore_shuttingdown())
+            {
+                break;
+            }
+        }
+
+        s64 wait_stop_begin = timeus();
+        bool wait_stop_error_message = FALSE;
+        for(u32 wait_count = 0; thread_pool_counter_get_value(&counter) > 0; ++wait_count)
         {
             sleep(1);
+            s64 wait_stop_now = timeus();
+            s64 wait_stop_duration = wait_stop_now - wait_stop_begin;
+            if(dnscore_shuttingdown() && (wait_stop_duration > (ONE_SECOND_US * 30)) )
+            {
+                if(!wait_stop_error_message)
+                {
+                    log_err("keyroll workers aren't stopping");
+                    logger_flush();
+                    wait_stop_error_message = TRUE;
+                }
+                if(wait_stop_duration > (ONE_SECOND_US * 60))
+                {
+                    log_err("not waiting anymore");
+                    logger_flush();
+                }
+            }
         }
 
         thread_pool_destroy(tp);
+        tp = NULL;
 
         free(args);
     }
 
+    if(does_loop)
+    {
+        log_info("keyroll stopped");
+    }
+
     unlink(pid_file_path);
 
     return SUCCESS;
