@@ -259,7 +259,7 @@ protocol_test_message_udp_with_timeout(message_data *mesg, const host_address *s
             }
             else
             {
-                ret = (n < 0)?ERRNO_ERROR:ERROR;
+                ret = (n < 0)?n:ERROR;
             }
             
             close_ex(sockfd);
@@ -381,7 +381,7 @@ hammer_message_udp_with_timeout(message_data *mesg, const host_address *server,
             }
             else
             {
-                ret = (n < 0)?ERRNO_ERROR:ERROR;
+                ret = (n < 0)?n:ERROR;
             }
             
             close_ex(sockfd);
@@ -1185,7 +1185,7 @@ query_z_opt_test(const host_address *server, const u8* fqdn)
         message_set_flags_hi(mesg, 0);
         message_set_flags_lo(mesg, 0);
 
-        message_set_opcode(mesg, OPCODE_QUERY << OPCODE_SHIFT);
+        message_set_opcode(mesg, OPCODE_QUERY); // note: OPCODE_QUERY is already shifted
         message_set_authenticated_data(mesg);
 
         message_set_query_answer_authority_additional_counts(mesg, 1, 0, 0, 1);
