@@ -606,7 +606,7 @@ int main(int argc, char *argv[])
     }
     
     dnscore_init();    
-    println("usage: %s [writer-count [insert-count-per-writer [thread-type-mask [message-size] ] ] ]");
+    formatln("usage: %s [writer-count [insert-count-per-writer [thread-type-mask [message-size] ] ] ]", argv[0]);
     formatln("running with %i %i %i %i", w, n, mask, msg_buffer_size);
 
     formatln("benching with %i items", n);
@@ -658,6 +658,7 @@ int main(int argc, char *argv[])
             }
 */
             thread_pool_destroy(tp);
+            tp = NULL;
         }
         else
         {
@@ -835,6 +836,7 @@ int main(int argc, char *argv[])
     }
 
     thread_pool_destroy(tp);
+    tp = NULL;
     threaded_ringbuffer_cw_finalize(&ringbuffer_cw_queue);
     threaded_dll_cw_finalize(&dll_cw_queue);
     threaded_qsl_cw_finalize(&qsl_cw_queue);
